@@ -34,6 +34,11 @@
 #include "gossip-protocol.h"
 #include "gossip-vcard.h"
 
+typedef enum {
+	GOSSIP_DISCONNECT_ASKED,
+	GOSSIP_DISCONNECT_ERROR
+} GossipDisconnectReason;
+
 #define GOSSIP_TYPE_SESSION         (gossip_session_get_type ())
 #define GOSSIP_SESSION(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_SESSION, GossipSession))
 #define GOSSIP_SESSION_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), GOSSIP_TYPE_SESSION, GossipSessionClass))
@@ -61,15 +66,15 @@ GossipProtocol *gossip_session_get_protocol            (GossipSession
 
 /* providers */
 GossipChatroomProvider *
-                gossip_session_get_chatroom_provider   (GossipSession           *session,
+		gossip_session_get_chatroom_provider   (GossipSession           *session,
 							GossipAccount           *account);
 GossipFTProvider *
-                gossip_session_get_ft_provider         (GossipSession           *session,
+		gossip_session_get_ft_provider         (GossipSession           *session,
 							GossipAccount           *account);
 
 /* accounts */
 GossipAccountManager *
-                gossip_session_get_account_manager     (GossipSession           *session);
+		gossip_session_get_account_manager     (GossipSession           *session);
 GList *         gossip_session_get_accounts            (GossipSession           *session);
 gdouble         gossip_session_get_connected_time      (GossipSession           *session,
 							GossipAccount           *account);
@@ -78,7 +83,8 @@ void            gossip_session_count_accounts          (GossipSession
 							guint                   *connected,
 							guint                   *connecting,
 							guint                   *disconnected);
-
+GossipAccount * gossip_session_new_account             (GossipSession           *session,
+							GossipAccountType        type);
 gboolean        gossip_session_add_account             (GossipSession           *session,
 							GossipAccount           *account);
 gboolean        gossip_session_remove_account          (GossipSession           *session,
