@@ -42,7 +42,8 @@
 #include "gossip-preferences.h"
 #include "gossip-theme-manager.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
+		       GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
 
 #define DEBUG_DOMAIN "ChatView"
 
@@ -69,9 +70,11 @@ struct _GossipChatViewPriv {
 
 	gboolean       allow_scrolling;
 	gboolean       is_group_chat;
-	
-	GtkTextMark   *find_mark;
+
+	GtkTextMark   *find_mark_previous;
+	GtkTextMark   *find_mark_next;
 	gboolean       find_wrapped;
+	gboolean       find_last_direction;
 
 	/* This is for the group chat so we know if the "other" last contact
 	 * changed, so we know whether to insert a header or not.
@@ -136,7 +139,7 @@ static const GossipSmileyPattern smileys[] = {
 	{ GOSSIP_SMILEY_MONKEY,       "#)"  },
 	{ GOSSIP_SMILEY_SILLY,        "O)"  },
 	{ GOSSIP_SMILEY_SICK,         "+o(" },
-	
+
 	/* Backward smileys. */
 	{ GOSSIP_SMILEY_NORMAL,       "(:"  },
 	{ GOSSIP_SMILEY_WINK,         "(;"  },
@@ -275,7 +278,7 @@ gossip_chat_view_init (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
 	gboolean            show_avatars;
-		
+
 	priv = GET_PRIV (view);
 
 	priv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
@@ -293,14 +296,14 @@ gossip_chat_view_init (GossipChatView *view)
 		      "cursor-visible", FALSE,
 		      NULL);
 
-	priv->notify_system_fonts_id = 
+	priv->notify_system_fonts_id =
 		gossip_conf_notify_add (gossip_conf_get (),
 					 "/desktop/gnome/interface/document_font_name",
 					 chat_view_notify_system_font_cb,
 					 view);
 	chat_view_system_font_update (view);
-	
-	priv->notify_show_avatars_id = 
+
+	priv->notify_show_avatars_id =
 		gossip_conf_notify_add (gossip_conf_get (),
 					 GOSSIP_PREFS_UI_SHOW_AVATARS,
 					 chat_view_notify_show_avatars_cb,
@@ -314,10 +317,10 @@ gossip_chat_view_init (GossipChatView *view)
 	gossip_conf_get_bool (gossip_conf_get (),
 			       GOSSIP_PREFS_UI_SHOW_AVATARS,
 			       &show_avatars);
-	
+
 	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
 						  view, show_avatars);
-	
+
 	g_signal_connect (view,
 			  "populate-popup",
 			  G_CALLBACK (chat_view_populate_popup),
@@ -359,7 +362,7 @@ chat_view_drag_motion (GtkWidget        *widget,
 	/* Don't handle drag motion, since we don't want the view to scroll as
 	 * the result of dragging something across it.
 	 */
-	
+
 	return TRUE;
 }
 
@@ -401,14 +404,14 @@ chat_view_setup_tags (GossipChatView *view)
 	tag = gtk_text_buffer_create_tag (priv->buffer,
 					  "link",
 					  NULL);
-	
+
 	g_signal_connect (tag,
 			  "event",
 			  G_CALLBACK (chat_view_url_event_cb),
 			  priv->buffer);
 
 	g_signal_connect (view,
-			  "event",
+			  "motion-notify-event",
 			  G_CALLBACK (chat_view_event_cb),
 			  tag);
 }
@@ -427,7 +430,7 @@ chat_view_system_font_update (GossipChatView *view)
 	} else {
 		font_description = NULL;
 	}
-	
+
 	gtk_widget_modify_font (GTK_WIDGET (view), font_description);
 
 	if (font_description) {
@@ -446,7 +449,7 @@ chat_view_notify_system_font_cb (GossipConf  *conf,
 	view = user_data;
 
 	chat_view_system_font_update (view);
-	
+
 	/* Ugly, again, to adjust the vertical position of the nick... Will fix
 	 * this when reworking the theme manager so that view register
 	 * themselves with it instead of the other way around.
@@ -454,7 +457,7 @@ chat_view_notify_system_font_cb (GossipConf  *conf,
 	gossip_conf_get_bool (conf,
 			       GOSSIP_PREFS_UI_SHOW_AVATARS,
 			       &show_avatars);
-	
+
 	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
 						  view, show_avatars);
 }
@@ -472,7 +475,7 @@ chat_view_notify_show_avatars_cb (GossipConf  *conf,
 	priv = GET_PRIV (view);
 
 	gossip_conf_get_bool (conf, key, &show_avatars);
-	
+
 	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
 						  view, show_avatars);
 }
@@ -497,11 +500,11 @@ chat_view_populate_popup (GossipChatView *view,
 		item = gtk_menu_item_new ();
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
-		
+
 		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
-		
+
 		g_signal_connect (item,
 				  "activate",
 				  G_CALLBACK (chat_view_clear_view_cb),
@@ -566,6 +569,7 @@ chat_view_event_cb (GossipChatView *view,
 		    GtkTextTag     *tag)
 {
 	static GdkCursor  *hand = NULL;
+	static GdkCursor  *beam = NULL;
 	GtkTextWindowType  type;
 	GtkTextIter        iter;
 	GdkWindow         *win;
@@ -598,10 +602,14 @@ chat_view_event_cb (GossipChatView *view,
 	if (gtk_text_iter_has_tag (&iter, tag)) {
 		if (!hand) {
 			hand = gdk_cursor_new (GDK_HAND2);
+			beam = gdk_cursor_new (GDK_XTERM);
 		}
 		gdk_window_set_cursor (win, hand);
 	} else {
-		gdk_window_set_cursor (win, NULL);
+		if (!beam) {
+			beam = gdk_cursor_new (GDK_XTERM);
+		}
+		gdk_window_set_cursor (win, beam);
 	}
 
 	return FALSE;
@@ -722,7 +730,7 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 					     smileys_index[i] > 0) &&
 					    smileys[i].pattern[smileys_index[i]] == c) {
 						submatch = i;
-						
+
 						smileys_index[i]++;
 						if (!smileys[i].pattern[smileys_index[i]]) {
 							match = i;
@@ -743,15 +751,15 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 		}
 
 		start = p - strlen (smileys[match].pattern);
-		
+
 		if (start > str) {
 			len = start - str;
 			gtk_text_buffer_insert (buf, iter, str, len);
 		}
-		
+
 		pixbuf = gossip_chat_view_get_smiley_image (smileys[match].smiley);
 		gtk_text_buffer_insert_pixbuf (buf, iter, pixbuf);
-		
+
 		gtk_text_buffer_insert (buf, iter, " ", 1);
 
 		str = g_utf8_find_next_char (p, NULL);
@@ -811,7 +819,7 @@ chat_view_maybe_trim_buffer (GossipChatView *view)
 	if (!tag) {
 		return;
 	}
-	
+
 	if (!gtk_text_iter_forward_to_tag_toggle (&bottom, tag)) {
 		return;
 	}
@@ -840,7 +848,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 	} else {
 		tag = "fancy-time";
 	}
-	
+
 	if (priv->last_block_type == BLOCK_TYPE_TIME) {
 		return;
 	}
@@ -851,7 +859,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 	if (msg) {
 		timestamp = gossip_message_get_timestamp (msg);
 	}
-	
+
 	if (timestamp <= 0) {
 		timestamp = gossip_time_get_current ();
 	}
@@ -869,7 +877,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 		append_date = TRUE;
 		append_time = TRUE;
 	}
-	
+
 	if (priv->last_timestamp + TIMESTAMP_INTERVAL < timestamp) {
 		append_time = TRUE;
 	}
@@ -879,7 +887,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 
 		g_string_append (str, "- ");
 	}
-	
+
 	if (append_date) {
 		gchar buf[256];
 
@@ -893,7 +901,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 
 	g_date_free (date);
 	g_date_free (last_date);
-	
+
 	if (append_time) {
 		gchar *tmp;
 
@@ -911,7 +919,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 							  str->str, -1,
 							  tag,
 							  NULL);
-		
+
 		priv->last_block_type = BLOCK_TYPE_TIME;
 		priv->last_timestamp = timestamp;
 	}
@@ -925,7 +933,7 @@ chat_view_append_spacing (GossipChatView *view)
 	GossipChatViewPriv *priv;
 	const gchar        *tag;
 	GtkTextIter         iter;
-	
+
 	priv = GET_PRIV (view);
 
 	if (priv->irc_style) {
@@ -933,7 +941,7 @@ chat_view_append_spacing (GossipChatView *view)
 	} else {
 		tag = "fancy-spacing";
 	}
-	
+
 	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
 						  &iter,
@@ -964,7 +972,7 @@ chat_view_append_text (GossipChatView *view,
 	} else {
 		link_tag = "fancy-link";
 	}
-	
+
 	gtk_text_buffer_get_end_iter (priv->buffer, &start_iter);
 	mark = gtk_text_buffer_create_mark (priv->buffer, NULL, &start_iter, TRUE);
 
@@ -1031,12 +1039,12 @@ chat_view_append_text (GossipChatView *view,
 	/* Apply the style to the inserted text. */
 	gtk_text_buffer_get_iter_at_mark (priv->buffer, &start_iter, mark);
 	gtk_text_buffer_get_end_iter (priv->buffer, &end_iter);
-	
+
 	gtk_text_buffer_apply_tag_by_name (priv->buffer,
 					   tag,
 					   &start_iter,
 					   &end_iter);
-	
+
 	gtk_text_buffer_delete_mark (priv->buffer, mark);
 }
 
@@ -1061,18 +1069,18 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 	priv = GET_PRIV (view);
 
 	contact = gossip_message_get_sender (msg);
-	
+
 	gossip_debug (DEBUG_DOMAIN, "Maybe add fancy header");
 
 	if (from_self) {
 		name = gossip_contact_get_name (my_contact);
-		
+
 		tag = "fancy-header-self";
 		line_top_tag = "fancy-line-top-self";
 		line_bottom_tag = "fancy-line-bottom-self";
 	} else {
 		name = gossip_contact_get_name (contact);
-		
+
 		tag = "fancy-header-other";
 		line_top_tag = "fancy-line-top-other";
 		line_bottom_tag = "fancy-line-bottom-other";
@@ -1093,18 +1101,18 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 	else if (!from_self && priv->last_block_type == BLOCK_TYPE_SELF) {
 		header = TRUE;
 	}
-	else if (!from_self && 
+	else if (!from_self &&
 		 (!priv->last_contact ||
 		  !gossip_contact_equal (contact, priv->last_contact))) {
 		header = TRUE;
 	}
-		
+
 	if (!header) {
 		return;
 	}
-	
+
 	chat_view_append_spacing (view);
-		
+
 	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
 						  &iter,
@@ -1115,9 +1123,9 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 
 	if (avatar) {
 		GtkTextIter start;
-		
+
 		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
- 		gtk_text_buffer_insert_pixbuf (priv->buffer, &iter, avatar);
+		gtk_text_buffer_insert_pixbuf (priv->buffer, &iter, avatar);
 
 		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 		start = iter;
@@ -1133,8 +1141,8 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 							   "fancy-avatar-other",
 							   &start, &iter);
 			avatar_tag = "fancy-header-other-avatar";
-		}			
-		
+		}
+
 	} else {
 		avatar_tag = NULL;
 	}
@@ -1180,14 +1188,14 @@ chat_view_append_irc_action (GossipChatView *view,
 	/* Skip the "/me ". */
 	if (from_self) {
 		name = gossip_contact_get_name (my_contact);
-		
+
 		tag = "irc-action-self";
 	} else {
 		GossipContact *contact;
 
 		contact = gossip_message_get_sender (msg);
 		name = gossip_contact_get_name (contact);
-		
+
 		tag = "irc-action-other";
 	}
 
@@ -1195,7 +1203,7 @@ chat_view_append_irc_action (GossipChatView *view,
 	    priv->last_block_type != BLOCK_TYPE_OTHER) {
 		chat_view_append_spacing (view);
 	}
-	
+
 	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 
 	tmp = g_strdup_printf (" * %s ", name);
@@ -1235,7 +1243,7 @@ chat_view_append_fancy_action (GossipChatView *view,
 
 	if (from_self) {
 		name = gossip_contact_get_name (my_contact);
-		
+
 		tag = "fancy-action-self";
 		line_tag = "fancy-line-self";
 	} else {
@@ -1243,7 +1251,7 @@ chat_view_append_fancy_action (GossipChatView *view,
 
 		contact = gossip_message_get_sender (msg);
 		name = gossip_contact_get_name (contact);
-		
+
 		tag = "fancy-action-other";
 		line_tag = "fancy-line-other";
 	}
@@ -1284,7 +1292,7 @@ chat_view_append_irc_message (GossipChatView *view,
 
 	if (from_self) {
 		name = gossip_contact_get_name (my_contact);
-		
+
 		nick_tag = "irc-nick-self";
 		body_tag = "irc-body-self";
 	} else {
@@ -1298,7 +1306,7 @@ chat_view_append_irc_message (GossipChatView *view,
 		} else {
 			nick_tag = "irc-nick-other";
 		}
-		
+
 		body_tag = "irc-body-other";
 	}
 
@@ -1335,7 +1343,7 @@ chat_view_append_fancy_message (GossipChatView *view,
 	const gchar        *tag;
 
 	priv = GET_PRIV (view);
-	
+
 	if (from_self) {
 		tag = "fancy-body-self";
 	} else {
@@ -1343,7 +1351,7 @@ chat_view_append_fancy_message (GossipChatView *view,
 
 		/* FIXME: Might want to support nick highlighting here... */
 	}
-	
+
 	body = gossip_message_get_body (msg);
 	chat_view_append_text (view, body, tag);
 }
@@ -1424,19 +1432,20 @@ chat_view_theme_changed_cb (GossipThemeManager *manager,
 {
 	GossipChatViewPriv *priv;
 	gboolean            show_avatars = FALSE;
+	gboolean            theme_rooms = FALSE;
 
 	priv = GET_PRIV (view);
 
-	/* Do nothing for group chats since they are always the same
-	 * theme.
-	 */
-	if (priv->is_group_chat) {
-		return;
-	}
-
 	priv->last_block_type = BLOCK_TYPE_NONE;
-	
-	gossip_theme_manager_apply_saved (manager, view);
+
+	gossip_conf_get_bool (gossip_conf_get (),
+			      GOSSIP_PREFS_CHAT_THEME_CHAT_ROOM,
+			      &theme_rooms);
+	if (!theme_rooms && priv->is_group_chat) {
+		gossip_theme_manager_apply (manager, view, NULL);
+	} else {
+		gossip_theme_manager_apply_saved (manager, view);
+	}
 
 	/* Needed for now to update the "rise" property of the names to get it
 	 * vertically centered.
@@ -1481,7 +1490,7 @@ gossip_chat_view_append_message_from_self (GossipChatView *view,
 						     my_contact,
 						     TRUE, avatar);
 	}
-	
+
 	/* Handle action messages (/me) and normal messages, in combination with
 	 * irc style and fancy style.
 	 */
@@ -1506,7 +1515,7 @@ gossip_chat_view_append_message_from_self (GossipChatView *view,
 		g_object_unref (priv->last_contact);
 		priv->last_contact = NULL;
 	}
-	
+
 	if (scroll_down) {
 		gossip_chat_view_scroll_down (view);
 	}
@@ -1540,7 +1549,7 @@ gossip_chat_view_append_message_from_other (GossipChatView *view,
 						     contact, FALSE,
 						     avatar);
 	}
-	
+
 	/* Handle action messages (/me) and normal messages, in combination with
 	 * irc style and fancy style.
 	 */
@@ -1565,7 +1574,7 @@ gossip_chat_view_append_message_from_other (GossipChatView *view,
 		g_object_unref (priv->last_contact);
 	}
 	priv->last_contact = g_object_ref (gossip_message_get_sender (msg));
-	
+
 	if (scroll_down) {
 		gossip_chat_view_scroll_down (view);
 	}
@@ -1582,7 +1591,7 @@ gossip_chat_view_append_event (GossipChatView *view,
 	const gchar        *tag;
 
 	priv = GET_PRIV (view);
-	
+
 	bottom = chat_view_is_scrolled_down (view);
 
 	chat_view_maybe_trim_buffer (view);
@@ -1594,7 +1603,7 @@ gossip_chat_view_append_event (GossipChatView *view,
 		tag = "fancy-event";
 		msg = g_strdup_printf (" - %s\n", str);
 	}
-	
+
 	if (priv->last_block_type != BLOCK_TYPE_EVENT) {
 		/* Comment out for now. */
 		/*chat_view_append_spacing (view);*/
@@ -1642,7 +1651,7 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	} else {
 		tag = "fancy-invite";
 	}
-	
+
 	bottom = chat_view_is_scrolled_down (view);
 
 	sender = gossip_message_get_sender (message);
@@ -1666,9 +1675,9 @@ gossip_chat_view_append_invite (GossipChatView *view,
 
 		s = g_string_append (s, reason);
 	}
-	
+
 	if (s->len < 1) {
-		s = g_string_append 
+		s = g_string_append
 			(s, _("You have been invited to join a chat conference."));
 	}
 
@@ -1683,7 +1692,7 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	}
 
 	s = g_string_prepend_c (s, '\n');
-	
+
 	chat_view_append_text (view, s->str, tag);
 	g_string_free (s, TRUE);
 
@@ -1716,10 +1725,10 @@ gossip_chat_view_append_invite (GossipChatView *view,
 
 	/* Set up data */
 	g_object_set_data_full (G_OBJECT (button_accept), "invite",
-				gossip_chatroom_invite_ref (invite), 
+				gossip_chatroom_invite_ref (invite),
 				(GDestroyNotify) gossip_chatroom_invite_unref);
- 	g_object_set_data_full (G_OBJECT (button_accept), "other_button",
- 				g_object_ref (button_decline), 
+	g_object_set_data_full (G_OBJECT (button_accept), "other_button",
+				g_object_ref (button_decline),
 				g_object_unref);
 
 	g_signal_connect (button_accept, "clicked",
@@ -1727,10 +1736,10 @@ gossip_chat_view_append_invite (GossipChatView *view,
 			  NULL);
 
 	g_object_set_data_full (G_OBJECT (button_decline), "invite",
-				gossip_chatroom_invite_ref (invite), 
+				gossip_chatroom_invite_ref (invite),
 				(GDestroyNotify) gossip_chatroom_invite_unref);
- 	g_object_set_data_full (G_OBJECT (button_decline), "other_button",
- 				g_object_ref (button_accept), 
+	g_object_set_data_full (G_OBJECT (button_decline), "other_button",
+				g_object_ref (button_accept),
 				g_object_unref);
 
 	g_signal_connect (button_decline, "clicked",
@@ -1830,15 +1839,15 @@ gossip_chat_view_clear (GossipChatView *view)
 	 * conversations start.
 	 */
 	priv = GET_PRIV (view);
-	
+
 	priv->last_block_type = BLOCK_TYPE_NONE;
 	priv->last_timestamp = 0;
 }
 
-void
-gossip_chat_view_find (GossipChatView *view, 
-		       const gchar    *search_criteria,
-		       gboolean        new_search)
+gboolean
+gossip_chat_view_find_previous (GossipChatView *view,
+				const gchar    *search_criteria,
+				gboolean        new_search)
 {
 	GossipChatViewPriv *priv;
 	GtkTextBuffer      *buffer;
@@ -1848,22 +1857,22 @@ gossip_chat_view_find (GossipChatView *view,
 	gboolean            found;
 	gboolean            from_start = FALSE;
 
-	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
-	g_return_if_fail (search_criteria != NULL);
+	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
+	g_return_val_if_fail (search_criteria != NULL, FALSE);
 
 	priv = GET_PRIV (view);
-	
+
 	buffer = priv->buffer;
-	
+
 	if (strlen (search_criteria) == 0) {
-		if (priv->find_mark) {
+		if (priv->find_mark_previous) {
 			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
-			
+
 			gtk_text_buffer_move_mark (buffer,
-						   priv->find_mark,
+						   priv->find_mark_previous,
 						   &iter_at_mark);
 			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-						      priv->find_mark,
+						      priv->find_mark_previous,
 						      0.0,
 						      TRUE,
 						      0.0,
@@ -1872,57 +1881,73 @@ gossip_chat_view_find (GossipChatView *view,
 						      &iter_at_mark,
 						      &iter_at_mark);
 		}
-		
-		return;
+
+		return FALSE;
 	}
-	
+
 	if (new_search) {
 		from_start = TRUE;
 	}
-     
-	if (priv->find_mark) {
+
+	if (priv->find_mark_previous) {
 		gtk_text_buffer_get_iter_at_mark (buffer,
 						  &iter_at_mark,
-						  priv->find_mark);
+						  priv->find_mark_previous);
 	} else {
-		gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		gtk_text_buffer_get_end_iter (buffer, &iter_at_mark);
 		from_start = TRUE;
 	}
-	
-	found = gossip_text_iter_forward_search (&iter_at_mark,
-						 search_criteria,
-						 &iter_match_start, 
-						 &iter_match_end,
-						 NULL);
-    
+
+	priv->find_last_direction = FALSE;
+
+	found = gossip_text_iter_backward_search (&iter_at_mark,
+						  search_criteria,
+						  &iter_match_start,
+						  &iter_match_end,
+						  NULL);
+
 	if (!found) {
+		gboolean result = FALSE;
+
 		if (from_start) {
-			return;
+			return result;
 		}
-	
+
 		/* Here we wrap around. */
 		if (!new_search && !priv->find_wrapped) {
 			priv->find_wrapped = TRUE;
-			gossip_chat_view_find (view, search_criteria, FALSE);
+			result = gossip_chat_view_find_previous (view, 
+								 search_criteria, 
+								 FALSE);
 			priv->find_wrapped = FALSE;
 		}
 
-		return;
+		return result;
 	}
-    
+
 	/* Set new mark and show on screen */
-	if (!priv->find_mark) {
-		priv->find_mark = gtk_text_buffer_create_mark (buffer, NULL,
-							       &iter_match_end,
-							       TRUE);
+	if (!priv->find_mark_previous) {
+		priv->find_mark_previous = gtk_text_buffer_create_mark (buffer, NULL,
+									&iter_match_start,
+									TRUE);
 	} else {
 		gtk_text_buffer_move_mark (buffer,
-					   priv->find_mark,
+					   priv->find_mark_previous,
+					   &iter_match_start);
+	}
+
+	if (!priv->find_mark_next) {
+		priv->find_mark_next = gtk_text_buffer_create_mark (buffer, NULL,
+								    &iter_match_end,
+								    TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark_next,
 					   &iter_match_end);
 	}
-	
+
 	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-				      priv->find_mark,
+				      priv->find_mark_previous,
 				      0.0,
 				      TRUE,
 				      0.5,
@@ -1930,10 +1955,181 @@ gossip_chat_view_find (GossipChatView *view,
 
 	gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_match_start);
 	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);
+
+	return TRUE;
+}
+
+gboolean
+gossip_chat_view_find_next (GossipChatView *view,
+			    const gchar    *search_criteria,
+			    gboolean        new_search)
+{
+	GossipChatViewPriv *priv;
+	GtkTextBuffer      *buffer;
+	GtkTextIter         iter_at_mark;
+	GtkTextIter         iter_match_start;
+	GtkTextIter         iter_match_end;
+	gboolean            found;
+	gboolean            from_start = FALSE;
+
+	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
+	g_return_val_if_fail (search_criteria != NULL, FALSE);
+
+	priv = GET_PRIV (view);
+
+	buffer = priv->buffer;
+
+	if (strlen (search_criteria) == 0) {
+		if (priv->find_mark_next) {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+
+			gtk_text_buffer_move_mark (buffer,
+						   priv->find_mark_next,
+						   &iter_at_mark);
+			gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+						      priv->find_mark_next,
+						      0.0,
+						      TRUE,
+						      0.0,
+						      0.0);
+			gtk_text_buffer_select_range (buffer,
+						      &iter_at_mark,
+						      &iter_at_mark);
+		}
+
+		return FALSE;
+	}
+
+	if (new_search) {
+		from_start = TRUE;
+	}
+
+	if (priv->find_mark_next) {
+		gtk_text_buffer_get_iter_at_mark (buffer,
+						  &iter_at_mark,
+						  priv->find_mark_next);
+	} else {
+		gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		from_start = TRUE;
+	}
+
+	priv->find_last_direction = TRUE;
+
+	found = gossip_text_iter_forward_search (&iter_at_mark,
+						 search_criteria,
+						 &iter_match_start,
+						 &iter_match_end,
+						 NULL);
+
+	if (!found) {
+		gboolean result = FALSE;
+
+		if (from_start) {
+			return result;
+		}
+
+		/* Here we wrap around. */
+		if (!new_search && !priv->find_wrapped) {
+			priv->find_wrapped = TRUE;
+			result = gossip_chat_view_find_next (view, 
+							     search_criteria, 
+							     FALSE);
+			priv->find_wrapped = FALSE;
+		}
+
+		return result;
+	}
+
+	/* Set new mark and show on screen */
+	if (!priv->find_mark_next) {
+		priv->find_mark_next = gtk_text_buffer_create_mark (buffer, NULL,
+							       &iter_match_end,
+							       TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark_next,
+					   &iter_match_end);
+	}
+
+	if (!priv->find_mark_previous) {
+		priv->find_mark_previous = gtk_text_buffer_create_mark (buffer, NULL,
+									&iter_match_start,
+									TRUE);
+	} else {
+		gtk_text_buffer_move_mark (buffer,
+					   priv->find_mark_previous,
+					   &iter_match_start);
+	}
+
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+				      priv->find_mark_next,
+				      0.0,
+				      TRUE,
+				      0.5,
+				      0.5);
+
+	gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_match_start);
+	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);
+
+	return TRUE;
+}
+
+
+void
+gossip_chat_view_find_abilities (GossipChatView *view,
+				 const gchar    *search_criteria,
+				 gboolean       *can_do_previous,
+				 gboolean       *can_do_next)
+{
+	GossipChatViewPriv *priv;
+	GtkTextBuffer      *buffer;
+	GtkTextIter         iter_at_mark;
+	GtkTextIter         iter_match_start;
+	GtkTextIter         iter_match_end;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (search_criteria != NULL);
+	g_return_if_fail (can_do_previous != NULL && can_do_next != NULL);
+
+	priv = GET_PRIV (view);
+
+	buffer = priv->buffer;
+
+	if (can_do_previous) {
+		if (priv->find_mark_previous) {
+			gtk_text_buffer_get_iter_at_mark (buffer,
+							  &iter_at_mark,
+							  priv->find_mark_previous);
+		} else {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		}
+		
+		*can_do_previous = gossip_text_iter_backward_search (&iter_at_mark,
+								     search_criteria,
+								     &iter_match_start,
+								     &iter_match_end,
+								     NULL);
+	}
+
+	if (can_do_next) {
+		if (priv->find_mark_next) {
+			gtk_text_buffer_get_iter_at_mark (buffer,
+							  &iter_at_mark,
+							  priv->find_mark_next);
+		} else {
+			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		}
+		
+		*can_do_next = gossip_text_iter_forward_search (&iter_at_mark,
+								search_criteria,
+								&iter_match_start,
+								&iter_match_end,
+								NULL);
+	}
 }
 
 void
-gossip_chat_view_highlight (GossipChatView *view, 
+gossip_chat_view_highlight (GossipChatView *view,
 			    const gchar    *text)
 {
 	GtkTextBuffer *buffer;
@@ -1947,12 +2143,12 @@ gossip_chat_view_highlight (GossipChatView *view,
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-    
+
 	gtk_text_buffer_get_start_iter (buffer, &iter);
 
 	gtk_text_buffer_get_bounds (buffer, &iter_start, &iter_end);
-	gtk_text_buffer_remove_tag_by_name (buffer, "highlight", 
-					    &iter_start, 
+	gtk_text_buffer_remove_tag_by_name (buffer, "highlight",
+					    &iter_start,
 					    &iter_end);
 
 	if (!text || strlen (text) < 1) {
@@ -1960,9 +2156,9 @@ gossip_chat_view_highlight (GossipChatView *view,
 	}
 
 	while (1) {
-		found = gossip_text_iter_forward_search (&iter, 
-							 text, 
-							 &iter_match_start, 
+		found = gossip_text_iter_forward_search (&iter,
+							 text,
+							 &iter_match_start,
 							 &iter_match_end,
 							 NULL);
 
@@ -1970,8 +2166,8 @@ gossip_chat_view_highlight (GossipChatView *view,
 			break;
 		}
 
-		gtk_text_buffer_apply_tag_by_name (buffer, "highlight", 
-						   &iter_match_start, 
+		gtk_text_buffer_apply_tag_by_name (buffer, "highlight",
+						   &iter_match_start,
 						   &iter_match_end);
 
 		iter = iter_match_end;
@@ -1997,7 +2193,7 @@ gboolean
 gossip_chat_view_get_irc_style (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
-	
+
 	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
 
 	priv = GET_PRIV (view);
@@ -2010,7 +2206,7 @@ gossip_chat_view_set_irc_style (GossipChatView *view,
 				gboolean        irc_style)
 {
 	GossipChatViewPriv *priv;
-	
+
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
 	priv = GET_PRIV (view);
@@ -2023,7 +2219,7 @@ gossip_chat_view_set_margin (GossipChatView *view,
 			     gint            margin)
 {
 	GossipChatViewPriv *priv;
-	
+
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
 	priv = GET_PRIV (view);
@@ -2057,7 +2253,7 @@ const gchar *
 gossip_chat_view_get_smiley_text (GossipSmiley smiley)
 {
 	gint i;
-	
+
 	for (i = 0; i < G_N_ELEMENTS (smileys); i++) {
 		if (smileys[i].smiley != smiley) {
 			continue;
@@ -2065,12 +2261,12 @@ gossip_chat_view_get_smiley_text (GossipSmiley smiley)
 
 		return smileys[i].pattern;
 	}
-	
+
 	return NULL;
 }
 
 GtkWidget *
-gossip_chat_view_get_smiley_menu (GCallback    callback, 
+gossip_chat_view_get_smiley_menu (GCallback    callback,
 				  gpointer     user_data,
 				  GtkTooltips *tooltips)
 {
@@ -2102,14 +2298,14 @@ gossip_chat_view_get_smiley_menu (GCallback    callback,
 
 		smiley_text = gossip_chat_view_get_smiley_text (i);
 
-		gtk_tooltips_set_tip (tooltips, 
-				      item, 
-				      smiley_text, 
+		gtk_tooltips_set_tip (tooltips,
+				      item,
+				      smiley_text,
 				      NULL);
 
 		g_object_set_data  (G_OBJECT (item), "smiley_text", (gpointer) smiley_text);
 		g_signal_connect (item, "activate", callback, user_data);
-		
+
 		if (x > 3) {
 			y++;
 			x = 0;
@@ -2123,24 +2319,32 @@ gossip_chat_view_get_smiley_menu (GCallback    callback,
 	return menu;
 }
 
+/* FIXME: Do we really need this? Better to do it internally only at setup time,
+ * we will never change it on the fly.
+ */
 void
 gossip_chat_view_set_is_group_chat (GossipChatView *view,
 				    gboolean        is_group_chat)
 {
 	GossipChatViewPriv *priv;
+	gboolean            theme_rooms = FALSE;
 
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
-	
+
 	priv = GET_PRIV (view);
 
 	priv->is_group_chat = is_group_chat;
 
-	if (is_group_chat) {
-		gossip_theme_manager_apply (gossip_theme_manager_get (), 
-					    view, 
+	gossip_conf_get_bool (gossip_conf_get (),
+			      GOSSIP_PREFS_CHAT_THEME_CHAT_ROOM,
+			      &theme_rooms);
+
+	if (!theme_rooms && is_group_chat) {
+		gossip_theme_manager_apply (gossip_theme_manager_get (),
+					    view,
 					    NULL);
 	} else {
-		gossip_theme_manager_apply_saved (gossip_theme_manager_get (), 
+		gossip_theme_manager_apply_saved (gossip_theme_manager_get (),
 						  view);
 	}
 }
