@@ -21,12 +21,16 @@
  */
 
 #include <config.h>
+
 #include <string.h>
 #include <stdlib.h>
+
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 #include <glib/gi18n.h>
 
+#include <libgossip/gossip-debug.h>
+#include <libgossip/gossip-conf.h>
+
 #include "gossip-app.h"
 #include "gossip-chat.h"
 #include "gossip-chat-window.h"
@@ -39,51 +43,55 @@
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT, GossipChatPriv))
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");   */
+#define DEBUG_DOMAIN "Chat"
 
-#define CHAT_DIR_CREATE_MODE    (S_IRUSR | S_IWUSR | S_IXUSR)
-#define CHAT_FILE_CREATE_MODE   (S_IRUSR | S_IWUSR)
+#define CHAT_DIR_CREATE_MODE  (S_IRUSR | S_IWUSR | S_IXUSR)
+#define CHAT_FILE_CREATE_MODE (S_IRUSR | S_IWUSR)
+
+#define MAX_INPUT_HEIGHT 150
 
 struct _GossipChatPriv {
 	GossipChatWindow *window;
 
 	GtkTooltips      *tooltips;
-	
+
 	/* Used to automatically shrink a window that has temporarily grown due
 	 * to long input.
 	 */
 	gint              padding_height;
 	gint              default_window_height;
 	gint              last_input_height;
+	gboolean          vscroll_visible;
 };
 
 typedef struct {
 	GossipChat  *chat;
-       	gchar       *word;
+	gchar       *word;
 
-       	GtkTextIter  start;
-       	GtkTextIter  end;
+	GtkTextIter  start;
+	GtkTextIter  end;
 } GossipChatSpell;
 
-static void      gossip_chat_class_init            (GossipChatClass *klass);
-static void      gossip_chat_init                  (GossipChat      *chat);
-static void      chat_finalize                     (GObject         *object);
-static void      chat_input_text_buffer_changed_cb (GtkTextBuffer   *buffer,
-						    GossipChat      *chat);
-static void      chat_text_view_size_allocate_cb   (GtkWidget       *widget,
-						    GtkAllocation   *allocation,
-						    GossipChat      *chat);
-static void      chat_text_populate_popup_cb       (GtkTextView     *view,
-						    GtkMenu         *menu,
-						    GossipChat      *chat);
-static void      chat_text_check_word_spelling_cb  (GtkMenuItem     *menuitem,
-						    GossipChatSpell *chat_spell);
-static GossipChatSpell *chat_spell_new             (GossipChat      *chat,
-						    const gchar     *word,
-						    GtkTextIter      start,
-						    GtkTextIter      end);
-static void      chat_spell_free                   (GossipChatSpell *chat_spell);
+static void             gossip_chat_class_init            (GossipChatClass *klass);
+static void             gossip_chat_init                  (GossipChat      *chat);
+static void             chat_finalize                     (GObject         *object);
+static void             chat_input_text_buffer_changed_cb (GtkTextBuffer   *buffer,
+							   GossipChat      *chat);
+static void             chat_text_view_scroll_hide_cb     (GtkWidget       *widget,
+							   GossipChat      *chat);
+static void             chat_text_view_size_allocate_cb   (GtkWidget       *widget,
+							   GtkAllocation   *allocation,
+							   GossipChat      *chat);
+static void             chat_text_populate_popup_cb       (GtkTextView     *view,
+							   GtkMenu         *menu,
+							   GossipChat      *chat);
+static void             chat_text_check_word_spelling_cb  (GtkMenuItem     *menuitem,
+							   GossipChatSpell *chat_spell);
+static GossipChatSpell *chat_spell_new                    (GossipChat      *chat,
+							   const gchar     *word,
+							   GtkTextIter      start,
+							   GtkTextIter      end);
+static void             chat_spell_free                   (GossipChatSpell *chat_spell);
 
 enum {
 	COMPOSING,
@@ -106,15 +114,15 @@ gossip_chat_class_init (GossipChatClass *klass)
 
 	object_class->finalize = chat_finalize;
 
-        chat_signals[COMPOSING] =
-                g_signal_new ("composing",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GossipChatClass, composing),
-                              NULL, NULL,
+	chat_signals[COMPOSING] =
+		g_signal_new ("composing",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GossipChatClass, composing),
+			      NULL, NULL,
 			      gossip_marshal_VOID__BOOLEAN,
-                              G_TYPE_NONE,
-                              1, G_TYPE_BOOLEAN);
+			      G_TYPE_NONE,
+			      1, G_TYPE_BOOLEAN);
 
 	chat_signals[NEW_MESSAGE] =
 		g_signal_new ("new-message",
@@ -171,8 +179,9 @@ gossip_chat_init (GossipChat *chat)
 	priv = GET_PRIV (chat);
 
 	priv->tooltips = gtk_tooltips_new ();
-	
+
 	priv->default_window_height = -1;
+	priv->vscroll_visible = FALSE;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 	g_signal_connect (buffer,
@@ -210,18 +219,19 @@ chat_finalize (GObject *object)
 }
 
 static void
-chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
+chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
+				   GossipChat    *chat)
 {
 	GossipChatPriv *priv;
 	GtkTextIter     start, end;
 	gchar          *str;
-	gboolean        spell_checker;
+	gboolean        spell_checker = FALSE;
 
 	priv = GET_PRIV (chat);
 
- 	spell_checker = gconf_client_get_bool (gossip_app_get_gconf_client (), 
-					       "/apps/gossip/conversation/enable_spell_checker", 
-					       NULL); 
+	gossip_conf_get_bool (gossip_conf_get (),
+			      GOSSIP_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+			      &spell_checker);
 
 	if (chat->is_first_char) {
 		GtkRequisition  req;
@@ -256,7 +266,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 	if (!gossip_spell_supported ()) {
 		return;
 	}
-	
+
 	/* NOTE: this is really inefficient, we shouldn't have to
 	   reiterate the whole buffer each time and check each work
 	   every time. */
@@ -277,18 +287,18 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 				break;
 			}
 
-			start = end; 
+			start = end;
 			gtk_text_iter_backward_word_start (&start);
 		}
-	
+
 		str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
 		/* spell check string */
-  		if (!gossip_chat_get_is_command (str)) { 
+		if (!gossip_chat_get_is_command (str)) {
 			correct = gossip_spell_check (str);
- 		} else { 
- 			correct = TRUE; 
- 		} 
+		} else {
+			correct = TRUE;
+		}
 
 		if (!correct) {
 			gtk_text_buffer_apply_tag_by_name (buffer, "misspelled", &start, &end);
@@ -299,7 +309,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 		g_free (str);
 
 		/* set start iter to the end iters position */
-		start = end; 
+		start = end;
 	}
 }
 
@@ -312,26 +322,46 @@ typedef struct {
 static gboolean
 chat_change_size_in_idle_cb (ChangeSizeData *data)
 {
-	gtk_window_resize (GTK_WINDOW (data->window), 
+	gtk_window_resize (GTK_WINDOW (data->window),
 			   data->width, data->height);
 
 	return FALSE;
 }
 
+static void
+chat_text_view_scroll_hide_cb (GtkWidget  *widget,
+			       GossipChat *chat)
+{
+	GossipChatPriv *priv;
+	GtkWidget      *sw;
+
+	priv = GET_PRIV (chat);
+
+	priv->vscroll_visible = FALSE;
+	g_signal_handlers_disconnect_by_func (widget, chat_text_view_scroll_hide_cb, chat);
+
+	sw = gtk_widget_get_parent (chat->input_text_view);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+					GTK_POLICY_NEVER,
+					GTK_POLICY_NEVER);
+	g_object_set (sw, "height-request", -1, NULL);
+}
+
 static void
 chat_text_view_size_allocate_cb (GtkWidget     *widget,
-		                 GtkAllocation *allocation,
+				 GtkAllocation *allocation,
 				 GossipChat    *chat)
 {
-	GossipChatPriv   *priv;
-	gint              width;
-	GtkWidget        *dialog;
-	ChangeSizeData   *data;
-	gint              window_height;
-	gint              new_height;
-	GtkAllocation    *view_allocation;
-	gint              current_height;
-	gint              diff;
+	GossipChatPriv *priv;
+	gint            width;
+	GtkWidget      *dialog;
+	ChangeSizeData *data;
+	gint            window_height;
+	gint            new_height;
+	GtkAllocation  *view_allocation;
+	gint            current_height;
+	gint            diff;
+	GtkWidget      *sw;
 
 	priv = GET_PRIV (chat);
 
@@ -339,6 +369,21 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 		return;
 	}
 
+	sw = gtk_widget_get_parent (widget);
+	if (sw->allocation.height >= MAX_INPUT_HEIGHT && !priv->vscroll_visible) {
+		GtkWidget *vscroll;
+
+		priv->vscroll_visible = TRUE;
+		gtk_widget_set_size_request (sw, -1, MAX_INPUT_HEIGHT);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+						GTK_POLICY_NEVER,
+						GTK_POLICY_AUTOMATIC);
+		vscroll = gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (sw));
+		g_signal_connect (vscroll, "hide",
+				  G_CALLBACK (chat_text_view_scroll_hide_cb),
+				  chat);
+	}
+
 	if (priv->last_input_height <= allocation->height) {
 		priv->last_input_height = allocation->height;
 		return;
@@ -346,7 +391,7 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 
 	diff = priv->last_input_height - allocation->height;
 	priv->last_input_height = allocation->height;
-	
+
 	view_allocation = &GTK_WIDGET (chat->view)->allocation;
 
 	dialog = gossip_chat_window_get_dialog (priv->window);
@@ -378,7 +423,7 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 }
 
 static void
-chat_insert_smiley_activate_cb (GtkWidget  *menuitem, 
+chat_insert_smiley_activate_cb (GtkWidget  *menuitem,
 				GossipChat *chat)
 {
 	GtkTextBuffer *buffer;
@@ -389,7 +434,7 @@ chat_insert_smiley_activate_cb (GtkWidget  *menuitem,
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 	gtk_text_buffer_get_end_iter (buffer, &iter);
-	gtk_text_buffer_insert (buffer, &iter, 
+	gtk_text_buffer_insert (buffer, &iter,
 				smiley, -1);
 }
 
@@ -410,18 +455,18 @@ chat_text_populate_popup_cb (GtkTextView *view,
 	GtkWidget       *smiley_menu;
 
 	priv = GET_PRIV (chat);
-	
+
 	/* Add the emoticon menu. */
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
-	
+
 	item = gtk_menu_item_new_with_mnemonic (_("Insert Smiley"));
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
 	smiley_menu = gossip_chat_view_get_smiley_menu (
-		G_CALLBACK (chat_insert_smiley_activate_cb), 
+		G_CALLBACK (chat_insert_smiley_activate_cb),
 		chat,
 		priv->tooltips);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), smiley_menu);
@@ -431,22 +476,22 @@ chat_text_populate_popup_cb (GtkTextView *view,
 	table = gtk_text_buffer_get_tag_table (buffer);
 
 	tag = gtk_text_tag_table_lookup (table, "misspelled");
-	
+
 	gtk_widget_get_pointer (GTK_WIDGET (view), &x, &y);
-	
-	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view), 
+
+	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
 					       GTK_TEXT_WINDOW_WIDGET,
 					       x, y,
 					       &x, &y);
-	
+
 	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (view), &iter, x, y);
 
 	start = end = iter;
-	
+
 	if (gtk_text_iter_backward_to_tag_toggle (&start, tag) &&
 	    gtk_text_iter_forward_to_tag_toggle (&end, tag)) {
-					
-		str = gtk_text_buffer_get_text (buffer, 
+
+		str = gtk_text_buffer_get_text (buffer,
 						&start, &end, FALSE);
 	}
 
@@ -456,14 +501,14 @@ chat_text_populate_popup_cb (GtkTextView *view,
 
 	chat_spell = chat_spell_new (chat, str, start, end);
 
-	g_object_set_data_full (G_OBJECT (menu), 
-				"chat_spell", chat_spell, 
+	g_object_set_data_full (G_OBJECT (menu),
+				"chat_spell", chat_spell,
 				(GDestroyNotify) chat_spell_free);
 
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
-	
+
 	item = gtk_menu_item_new_with_mnemonic (_("_Check Word Spelling..."));
 	g_signal_connect (item,
 			  "activate",
@@ -473,8 +518,8 @@ chat_text_populate_popup_cb (GtkTextView *view,
 	gtk_widget_show (item);
 }
 
-static void     
-chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem, 
+static void
+chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem,
 				  GossipChatSpell *chat_spell)
 {
 	gossip_spell_dialog_show (chat_spell->chat,
@@ -484,7 +529,7 @@ chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem,
 }
 
 static GossipChatSpell *
-chat_spell_new (GossipChat  *chat, 
+chat_spell_new (GossipChat  *chat,
 		const gchar *word,
 		GtkTextIter  start,
 		GtkTextIter  end)
@@ -497,7 +542,7 @@ chat_spell_new (GossipChat  *chat,
 	chat_spell->word = g_strdup (word);
 	chat_spell->start = start;
 	chat_spell->end = end;
-	
+
 	return chat_spell;
 }
 
@@ -517,7 +562,7 @@ gossip_chat_get_is_command (const gchar *str)
 	if (str[0] != '/') {
 		return FALSE;
 	}
-	
+
 	if (g_str_has_prefix (str, "/me")) {
 		return TRUE;
 	}
@@ -545,8 +590,8 @@ gossip_chat_correct_word (GossipChat  *chat,
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 
 	gtk_text_buffer_delete (buffer, &start, &end);
-	gtk_text_buffer_insert (buffer, &start, 
-				new_word, 
+	gtk_text_buffer_insert (buffer, &start,
+				new_word,
 				-1);
 }
 
@@ -646,7 +691,7 @@ gossip_chat_is_group_chat (GossipChat *chat)
 	return FALSE;
 }
 
-gboolean 
+gboolean
 gossip_chat_get_show_contacts (GossipChat *chat)
 {
 	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), FALSE);
@@ -658,8 +703,8 @@ gossip_chat_get_show_contacts (GossipChat *chat)
 	return FALSE;
 }
 
-void 
-gossip_chat_set_show_contacts (GossipChat *chat, 
+void
+gossip_chat_set_show_contacts (GossipChat *chat,
 			       gboolean    show)
 {
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
@@ -669,8 +714,8 @@ gossip_chat_set_show_contacts (GossipChat *chat,
 	}
 }
 
-void 
-gossip_chat_save_geometry (GossipChat *chat, 
+void
+gossip_chat_save_geometry (GossipChat *chat,
 			   gint        x,
 			   gint        y,
 			   gint        w,
@@ -679,7 +724,7 @@ gossip_chat_save_geometry (GossipChat *chat,
 	gossip_geometry_save_for_chat (chat, x, y, w, h);
 }
 
-void 
+void
 gossip_chat_load_geometry (GossipChat *chat,
 			   gint       *x,
 			   gint       *y,
@@ -698,7 +743,7 @@ gossip_chat_clear (GossipChat *chat)
 }
 
 void
-gossip_chat_set_window (GossipChat       *chat, 
+gossip_chat_set_window (GossipChat       *chat,
 			GossipChatWindow *window)
 {
 	GossipChatPriv *priv;
@@ -729,7 +774,7 @@ void
 gossip_chat_cut (GossipChat *chat)
 {
 	GtkTextBuffer *buffer;
-	
+
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
@@ -746,7 +791,7 @@ void
 gossip_chat_copy (GossipChat *chat)
 {
 	GtkTextBuffer *buffer;
-	
+
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
 	if (gossip_chat_view_get_selection_bounds (chat->view, NULL, NULL)) {
@@ -786,7 +831,7 @@ gossip_chat_present (GossipChat *chat)
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
 	priv = GET_PRIV (chat);
-	
+
 	if (priv->window == NULL) {
 		GossipChatWindow *window;
 		gboolean          for_group_chat;
@@ -802,11 +847,12 @@ gossip_chat_present (GossipChat *chat)
 		}
 
 		gossip_chat_window_add_chat (window, chat);
-        }
+	}
 
-        gossip_chat_window_switch_to_chat (priv->window, chat);
+	gossip_chat_window_switch_to_chat (priv->window, chat);
 	gossip_window_present (
-		GTK_WINDOW (gossip_chat_window_get_dialog (priv->window)));
+		GTK_WINDOW (gossip_chat_window_get_dialog (priv->window)),
+		TRUE);
 
 	gtk_widget_grab_focus (chat->input_text_view);
 }
@@ -821,7 +867,7 @@ gossip_chat_should_play_sound (GossipChat *chat)
 
 	window = gossip_chat_get_window (GOSSIP_CHAT (chat));
 	if (!window) {
-		return TRUE; 
+		return TRUE;
 	}
 
 	play = !gossip_chat_window_has_focus (window);
@@ -841,7 +887,7 @@ gossip_chat_should_highlight_nick (GossipMessage *message,
 
 	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), FALSE);
 
-	DEBUG_MSG (("Chat: Highlighting nickname"));
+	gossip_debug (DEBUG_DOMAIN, "Highlighting nickname");
 
 	ret_val = FALSE;
 
@@ -849,7 +895,7 @@ gossip_chat_should_highlight_nick (GossipMessage *message,
 	if (!msg) {
 		return FALSE;
 	}
-		
+
 	if (my_contact) {
 		contact = my_contact;
 	} else {
