@@ -36,16 +36,16 @@ typedef struct {
 	GtkWidget       *treeview;
 	GtkWidget       *button_ok;
 
-        GossipContact   *contact;
+	GossipContact   *contact;
 	GtkCellRenderer *renderer;
 	gboolean         changes_made;
 } GossipEditContactDialog;
 
 typedef struct {
 	GossipEditContactDialog *dialog;
-	
+
 	const gchar             *name;
-	
+
 	gboolean                 found;
 	GtkTreeIter              found_iter;
 } FindName;
@@ -62,38 +62,43 @@ enum {
 	COL_COUNT
 };
 
-static void     edit_contact_dialog_init                        (void);
-static void     edit_contact_dialog_save_name                   (GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_save_groups                 (GossipEditContactDialog *dialog);
-static gboolean edit_contact_dialog_can_save                    (GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_model_setup                 (GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_model_populate_columns      (GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_model_populate_data         (GossipEditContactDialog *dialog,
-								 GList                   *groups);
-static gboolean edit_contact_dialog_model_find_name             (GossipEditContactDialog *dialog,
-								 const gchar             *name,
-								 GtkTreeIter             *iter);
-static gboolean edit_contact_dialog_model_find_name_foreach     (GtkTreeModel            *model,
-								 GtkTreePath             *path,
-								 GtkTreeIter             *iter,
-								 FindName                *data);
-static GList *  edit_contact_dialog_model_find_selected         (GossipEditContactDialog *dialog);
-static gboolean edit_contact_dialog_model_find_selected_foreach (GtkTreeModel            *model,
-								 GtkTreePath             *path,
-								 GtkTreeIter             *iter,
-								 FindSelected            *data);
-static void     edit_contact_dialog_cell_toggled                (GtkCellRendererToggle   *cell,
-								 gchar                   *path_string,
-								 GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_entry_name_changed_cb       (GtkEditable             *editable,
-								 GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_button_add_clicked_cb       (GtkButton               *button,
-								 GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_destroy_cb                  (GtkWidget               *widget,
-								 GossipEditContactDialog *dialog);
-static void     edit_contact_dialog_response_cb                 (GtkWidget               *widget,
-								 gint                     response,
-								 GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_init                         (void);
+static void     edit_contact_dialog_save_name                    (GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_save_groups                  (GossipEditContactDialog *dialog);
+static gboolean edit_contact_dialog_can_save                     (GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_model_setup                  (GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_model_populate_columns       (GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_model_populate_data          (GossipEditContactDialog *dialog,
+								  GList                   *groups);
+static gboolean edit_contact_dialog_model_find_name              (GossipEditContactDialog *dialog,
+								  const gchar             *name,
+								  GtkTreeIter             *iter);
+static gboolean edit_contact_dialog_model_find_name_foreach      (GtkTreeModel            *model,
+								  GtkTreePath             *path,
+								  GtkTreeIter             *iter,
+								  FindName                *data);
+static GList *  edit_contact_dialog_model_find_selected          (GossipEditContactDialog *dialog);
+static gboolean edit_contact_dialog_model_find_selected_foreach  (GtkTreeModel            *model,
+								  GtkTreePath             *path,
+								  GtkTreeIter             *iter,
+								  FindSelected            *data);
+static void     edit_contact_dialog_cell_toggled                 (GtkCellRendererToggle   *cell,
+								  gchar                   *path_string,
+								  GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_entry_name_changed_cb        (GtkEditable             *editable,
+								  GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_button_add_clicked_cb        (GtkButton               *button,
+								  GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_button_retrieve_get_vcard_cb (GossipResult             result,
+								  GossipVCard             *vcard,
+								  GossipContact           *contact);
+static void     edit_contact_dialog_button_retrieve_clicked_cb   (GtkButton               *button,
+								  GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_destroy_cb                   (GtkWidget               *widget,
+								  GossipEditContactDialog *dialog);
+static void     edit_contact_dialog_response_cb                  (GtkWidget               *widget,
+								  gint                     response,
+								  GossipEditContactDialog *dialog);
 
 static GHashTable *dialogs = NULL;
 
@@ -103,7 +108,7 @@ edit_contact_dialog_init (void)
 	if (dialogs) {
 		return;
 	}
-	
+
 	dialogs = g_hash_table_new_full (gossip_contact_hash,
 					 gossip_contact_equal,
 					 g_object_unref,
@@ -117,8 +122,8 @@ edit_contact_dialog_save_name (GossipEditContactDialog *dialog)
 
 	name = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
 
-	gossip_session_rename_contact (gossip_app_get_session (), 
-				       dialog->contact, 
+	gossip_session_rename_contact (gossip_app_get_session (),
+				       dialog->contact,
 				       name);
 }
 
@@ -127,17 +132,17 @@ edit_contact_dialog_save_groups (GossipEditContactDialog *dialog)
 {
 	GList *groups;
 
-	groups = edit_contact_dialog_model_find_selected (dialog); 
+	groups = edit_contact_dialog_model_find_selected (dialog);
 
-        gossip_contact_set_groups (dialog->contact, groups);
-        gossip_session_update_contact (gossip_app_get_session (),
-                                       dialog->contact);
-	
-	g_list_foreach (groups, (GFunc)g_free, NULL); 
+	gossip_contact_set_groups (dialog->contact, groups);
+	gossip_session_update_contact (gossip_app_get_session (),
+				       dialog->contact);
+
+	g_list_foreach (groups, (GFunc)g_free, NULL);
 	g_list_free (groups);
 }
 
-static void 
+static void
 edit_contact_dialog_model_setup (GossipEditContactDialog *dialog)
 {
 	GtkTreeView      *view;
@@ -150,56 +155,56 @@ edit_contact_dialog_model_setup (GossipEditContactDialog *dialog)
 				    G_TYPE_STRING,   /* name */
 				    G_TYPE_BOOLEAN,  /* enabled */
 				    G_TYPE_BOOLEAN); /* editable */
-	
+
 	gtk_tree_view_set_model (view, GTK_TREE_MODEL (store));
 
 	selection = gtk_tree_view_get_selection (view);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-	
+
 	edit_contact_dialog_model_populate_columns (dialog);
 
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
 					      COL_NAME, GTK_SORT_ASCENDING);
 
 	g_object_unref (store);
 }
 
-static void 
+static void
 edit_contact_dialog_model_populate_columns (GossipEditContactDialog *dialog)
 {
 	GtkTreeView       *view;
 	GtkTreeModel      *model;
-	GtkTreeViewColumn *column; 
+	GtkTreeViewColumn *column;
 	GtkCellRenderer   *renderer;
 	guint              col_offset;
-	
+
 	view = GTK_TREE_VIEW (dialog->treeview);
 	model = gtk_tree_view_get_model (view);
-	
+
 	renderer = gtk_cell_renderer_toggle_new ();
-	g_signal_connect (renderer, "toggled", 
-			  G_CALLBACK (edit_contact_dialog_cell_toggled), 
+	g_signal_connect (renderer, "toggled",
+			  G_CALLBACK (edit_contact_dialog_cell_toggled),
 			  dialog);
-	
+
 	column = gtk_tree_view_column_new_with_attributes (_("Select"), renderer,
 							   "active", COL_ENABLED,
 							   NULL);
-	
+
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (column, 50);
 	gtk_tree_view_append_column (view, column);
-	
+
 	renderer = gtk_cell_renderer_text_new ();
 	col_offset = gtk_tree_view_insert_column_with_attributes (view,
 								  -1, _("Group"),
-								  renderer, 
+								  renderer,
 								  "text", COL_NAME,
 /* 								  "editable", COL_EDITABLE, */
 								  NULL);
-	
+
 	g_object_set_data (G_OBJECT (renderer),
 			   "column", GINT_TO_POINTER (COL_NAME));
-	
+
 	column = gtk_tree_view_get_column (view, col_offset - 1);
 	gtk_tree_view_column_set_sort_column_id (column, COL_NAME);
 	gtk_tree_view_column_set_resizable (column,FALSE);
@@ -213,12 +218,12 @@ edit_contact_dialog_model_populate_columns (GossipEditContactDialog *dialog)
 }
 
 static void
-edit_contact_dialog_model_populate_data (GossipEditContactDialog *dialog, 
+edit_contact_dialog_model_populate_data (GossipEditContactDialog *dialog,
 					 GList                   *groups)
 {
 	GtkTreeView  *view;
 	GtkListStore *store;
-	GtkTreeIter   iter;	
+	GtkTreeIter   iter;
 	GList        *l;
 	GList        *my_groups = NULL;
 	GList        *all_groups;
@@ -234,145 +239,145 @@ edit_contact_dialog_model_populate_data (GossipEditContactDialog *dialog,
 		if (strcmp (group_str, _("Unsorted")) == 0) {
 			continue;
 		}
-		
+
 		my_groups = g_list_append (my_groups, g_strdup (group_str));
 	}
-	
+
 	for (l = all_groups; l; l = l->next) {
 		const gchar *group_str = (const gchar *) l->data;
 
 		if (strcmp (group_str, _("Unsorted")) == 0) {
 			continue;
 		}
-	
+
 		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,    
+		gtk_list_store_set (store, &iter,
 				    COL_NAME, group_str,
 				    COL_EDITABLE, TRUE,
 				    -1);
 
 		if (g_list_find_custom (my_groups, group_str, (GCompareFunc)strcmp)) {
 			if (edit_contact_dialog_model_find_name (dialog, group_str, &iter)) {
-				gtk_list_store_set (store, &iter,    
+				gtk_list_store_set (store, &iter,
 						    COL_ENABLED, TRUE,
 						    -1);
 			}
 		}
 	}
 
-	g_list_foreach (my_groups, (GFunc) g_free, NULL); 
+	g_list_foreach (my_groups, (GFunc) g_free, NULL);
 	g_list_free (my_groups);
 
 	g_list_free (all_groups);
 }
 
-static gboolean 
-edit_contact_dialog_model_find_name (GossipEditContactDialog *dialog, 
+static gboolean
+edit_contact_dialog_model_find_name (GossipEditContactDialog *dialog,
 				     const gchar             *name,
 				     GtkTreeIter             *iter)
 {
 	GtkTreeView  *view;
 	GtkTreeModel *model;
 	FindName      data;
-	
+
 	if (!name || strlen (name) < 1) {
 		return FALSE;
 	}
-	
+
 	data.dialog = dialog;
 	data.name = name;
 	data.found = FALSE;
-	
+
 	view = GTK_TREE_VIEW (dialog->treeview);
 	model = gtk_tree_view_get_model (view);
-	
+
 	gtk_tree_model_foreach (model,
 				(GtkTreeModelForeachFunc) edit_contact_dialog_model_find_name_foreach,
 				&data);
-	
+
 	if (data.found == TRUE) {
 		*iter = data.found_iter;
 		return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
-static gboolean 
-edit_contact_dialog_model_find_selected_foreach (GtkTreeModel *model, 
-						 GtkTreePath  *path, 
-						 GtkTreeIter  *iter, 
-						 FindSelected *data) 
+static gboolean
+edit_contact_dialog_model_find_selected_foreach (GtkTreeModel *model,
+						 GtkTreePath  *path,
+						 GtkTreeIter  *iter,
+						 FindSelected *data)
 {
 	gchar    *name;
 	gboolean  selected;
 
-	gtk_tree_model_get (model, iter, 
-			    COL_NAME, &name, 
-			    COL_ENABLED, &selected, 
+	gtk_tree_model_get (model, iter,
+			    COL_NAME, &name,
+			    COL_ENABLED, &selected,
 			    -1);
-  
+
 	if (!name) {
 		return FALSE;
 	}
-	
+
 	if (selected) {
 		data->list = g_list_append (data->list, name);
 		return FALSE;
 	}
-	
+
 	g_free (name);
-  
+
 	return FALSE;
 }
 
-static gboolean 
-edit_contact_dialog_model_find_name_foreach (GtkTreeModel *model, 
-					     GtkTreePath  *path, 
-					     GtkTreeIter  *iter, 
-					     FindName     *data) 
+static gboolean
+edit_contact_dialog_model_find_name_foreach (GtkTreeModel *model,
+					     GtkTreePath  *path,
+					     GtkTreeIter  *iter,
+					     FindName     *data)
 {
 	gchar *name;
 
-	gtk_tree_model_get (model, iter, 
-			    COL_NAME, &name, 
+	gtk_tree_model_get (model, iter,
+			    COL_NAME, &name,
 			    -1);
-  
+
 	if (!name) {
 		return FALSE;
 	}
-	
+
 	if (data->name && strcmp (data->name, name) == 0) {
 		data->found = TRUE;
 		data->found_iter = *iter;
 
 		g_free (name);
-		
+
 		return TRUE;
 	}
 
 	g_free (name);
-	
+
 	return FALSE;
 }
 
-static GList * 
+static GList *
 edit_contact_dialog_model_find_selected (GossipEditContactDialog *dialog)
 {
 	GtkTreeView  *view;
 	GtkTreeModel *model;
 	FindSelected  data;
-	
+
 	data.dialog = dialog;
 	data.list = NULL;
 
 	view = GTK_TREE_VIEW (dialog->treeview);
 	model = gtk_tree_view_get_model (view);
-	
+
 	gtk_tree_model_foreach (model,
 				(GtkTreeModelForeachFunc) edit_contact_dialog_model_find_selected_foreach,
 				&data);
-	
+
 	return data.list;
 }
 
@@ -391,9 +396,9 @@ edit_contact_dialog_can_save (GossipEditContactDialog *dialog)
 	return ok;
 }
 
-static void 
-edit_contact_dialog_cell_toggled (GtkCellRendererToggle   *cell, 
-				  gchar                   *path_string, 
+static void
+edit_contact_dialog_cell_toggled (GtkCellRendererToggle   *cell,
+				  gchar                   *path_string,
 				  GossipEditContactDialog *dialog)
 {
 	GtkTreeView  *view;
@@ -409,7 +414,7 @@ edit_contact_dialog_cell_toggled (GtkCellRendererToggle   *cell,
 	view = GTK_TREE_VIEW (dialog->treeview);
 	model = gtk_tree_view_get_model (view);
 	store = GTK_LIST_STORE (model);
-	
+
 	path = gtk_tree_path_new_from_string (path_string);
 
 	gtk_tree_model_get_iter (model, &iter, path);
@@ -421,7 +426,7 @@ edit_contact_dialog_cell_toggled (GtkCellRendererToggle   *cell,
 	gtk_tree_path_free (path);
 
 	dialog->changes_made = TRUE;
-	
+
 	gtk_widget_set_sensitive (dialog->button_ok,
 				  edit_contact_dialog_can_save (dialog));
 }
@@ -440,47 +445,47 @@ edit_contact_dialog_entry_name_changed_cb (GtkEditable             *editable,
 				  edit_contact_dialog_can_save (dialog));
 }
 
-static void 
-edit_contact_dialog_entry_group_changed_cb (GtkEditable             *editable, 
+static void
+edit_contact_dialog_entry_group_changed_cb (GtkEditable             *editable,
 					    GossipEditContactDialog *dialog)
 {
 	GtkTreeIter  iter;
 	const gchar *group;
-	
+
 	group = gtk_entry_get_text (GTK_ENTRY (dialog->entry_group));
-	
+
 	if (edit_contact_dialog_model_find_name (dialog, group, &iter)) {
 		gtk_widget_set_sensitive (GTK_WIDGET (dialog->button_add), FALSE);
-	
+
 	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (dialog->button_add), 
+		gtk_widget_set_sensitive (GTK_WIDGET (dialog->button_add),
 					  group && strlen (group) > 0);
 	}
 }
 
-static void 
-edit_contact_dialog_entry_group_activate_cb (GtkEntry                *entry, 
+static void
+edit_contact_dialog_entry_group_activate_cb (GtkEntry                *entry,
 					     GossipEditContactDialog *dialog)
 {
 	gtk_widget_activate (GTK_WIDGET (dialog->button_add));
 }
 
-static void 
-edit_contact_dialog_button_add_clicked_cb (GtkButton               *button, 
+static void
+edit_contact_dialog_button_add_clicked_cb (GtkButton               *button,
 					   GossipEditContactDialog *dialog)
 {
 	GtkTreeView  *view;
 	GtkListStore *store;
 	GtkTreeIter   iter;
 	const gchar  *group;
-	
+
 	view = GTK_TREE_VIEW (dialog->treeview);
 	store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
 
 	group = gtk_entry_get_text (GTK_ENTRY (dialog->entry_group));
 
 	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter,    
+	gtk_list_store_set (store, &iter,
 			    COL_NAME, group,
 			    COL_ENABLED, TRUE,
 			    -1);
@@ -492,20 +497,75 @@ edit_contact_dialog_button_add_clicked_cb (GtkButton               *button,
 }
 
 static void
-edit_contact_dialog_response_cb (GtkWidget               *widget, 
-				 gint                     response, 
+edit_contact_dialog_button_retrieve_get_vcard_cb (GossipResult   result,
+						  GossipVCard   *vcard,
+						  GossipContact *contact)
+{
+	GossipEditContactDialog *dialog;
+	const gchar             *name;
+
+	dialog = g_hash_table_lookup (dialogs, contact);
+	g_object_unref (contact);
+
+	if (!dialog) {
+		return;
+	}
+
+	gtk_widget_set_sensitive (dialog->entry_name, TRUE);
+	gtk_widget_set_sensitive (dialog->button_retrieve, TRUE);
+
+	if (result != GOSSIP_RESULT_OK) {
+		return;
+	}
+
+	name = gossip_vcard_get_nickname (vcard);
+	if (name && strlen (name) > 0) {
+		gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), name);
+		return;
+	}
+
+	name = gossip_vcard_get_name (vcard);
+	if (name && strlen (name) > 0) {
+		gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), name);
+		return;
+	}
+}
+
+static void
+edit_contact_dialog_button_retrieve_clicked_cb (GtkButton               *button,
+						GossipEditContactDialog *dialog)
+{
+	GossipSession *session;
+
+	gtk_widget_set_sensitive (dialog->entry_name, FALSE);
+	gtk_widget_set_sensitive (dialog->button_retrieve, FALSE);
+
+	session = gossip_app_get_session ();
+
+	gossip_session_get_vcard (session,
+				  NULL,
+				  dialog->contact,
+				  (GossipVCardCallback)
+				  edit_contact_dialog_button_retrieve_get_vcard_cb,
+				  g_object_ref (dialog->contact),
+				  NULL);
+}
+
+static void
+edit_contact_dialog_response_cb (GtkWidget               *widget,
+				 gint                     response,
 				 GossipEditContactDialog *dialog)
 {
 	if (dialog->changes_made && response == GTK_RESPONSE_OK) {
 		edit_contact_dialog_save_groups (dialog);
 		edit_contact_dialog_save_name (dialog);
 	}
-	
+
 	gtk_widget_destroy (dialog->dialog);
 }
 
 static void
-edit_contact_dialog_destroy_cb (GtkWidget        *widget,
+edit_contact_dialog_destroy_cb (GtkWidget               *widget,
 				GossipEditContactDialog *dialog)
 {
 	if (dialog->renderer) {
@@ -514,7 +574,7 @@ edit_contact_dialog_destroy_cb (GtkWidget        *widget,
 
 	g_hash_table_remove (dialogs, dialog->contact);
 
- 	g_free (dialog); 
+	g_free (dialog);
 }
 
 void
@@ -525,7 +585,7 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 	GladeXML                *glade;
 	gchar                   *id;
 	gchar                   *str;
-        GList                   *groups;
+	GList                   *groups;
 	GtkSizeGroup            *size_group;
 
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
@@ -540,10 +600,10 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 
 	dialog = g_new0 (GossipEditContactDialog, 1);
 
-        dialog->contact = g_object_ref (contact);
+	dialog->contact = g_object_ref (contact);
 
 	g_hash_table_insert (dialogs, dialog->contact, dialog);
-        
+
 	glade = gossip_glade_get_file ("main.glade",
 				       "edit_contact_dialog",
 				       NULL,
@@ -565,15 +625,16 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 			      "entry_group", "changed", edit_contact_dialog_entry_group_changed_cb,
 			      "entry_group", "activate", edit_contact_dialog_entry_group_activate_cb,
 			      "button_add", "clicked", edit_contact_dialog_button_add_clicked_cb,
+			      "button_retrieve", "clicked", edit_contact_dialog_button_retrieve_clicked_cb,
 			      NULL);
 
 	g_object_unref (glade);
 
 	/* Set up name */
-	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), 
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name),
 			    gossip_contact_get_name (dialog->contact));
 	gtk_editable_select_region (GTK_EDITABLE (dialog->entry_name), 0, -1);
-	
+
 	id = g_markup_escape_text (gossip_contact_get_id (contact), -1);
 	str = g_strdup_printf (_("Set the alias you want to use for:\n"
 				 "<b>%s</b>\n"
@@ -585,10 +646,8 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 	gtk_label_set_markup (GTK_LABEL (dialog->label_name), str);
 	g_free (str);
 
-	gtk_widget_set_sensitive (dialog->button_retrieve, FALSE);
-
 	/* Set up groups */
-        groups = gossip_contact_get_groups (contact);
+	groups = gossip_contact_get_groups (contact);
 
 	edit_contact_dialog_model_setup (dialog);
 	edit_contact_dialog_model_populate_data (dialog, groups);
@@ -601,7 +660,7 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 
 	/* Set up transient parent */
 	if (parent) {
-		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent); 
+		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent);
 	}
 
 	gtk_widget_show_all (dialog->dialog);
