@@ -19,79 +19,65 @@
  */
 
 /* Part of this file is copied from GtkSourceView (gtksourceiter.c):
- *  Copyright (C) 2000 - 2005 Paolo Maggi 
+ *  Copyright (C) 2000 - 2005 Paolo Maggi
  *  Copyright (C) 2002, 2003 Jeroen Zwartepoorte
  */
 
 #include <config.h>
 #include <string.h>
-#include <time.h>
-#include <sys/types.h>
-#include <regex.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
+
+#ifdef HAVE_GNOME
+#include <gdk/gdkx.h>
 #include <libgnome/gnome-url.h>
-#include <glib/gi18n.h>
+#include <libgnomeui/libgnomeui.h>
+#elif defined (HAVE_COCOA)
+#include <Cocoa/Cocoa.h>
+#endif
+
+#ifdef HAVE_LIBNOTIFY
+#include "gossip-notify.h"
+#endif
+
+#include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-paths.h>
+
 #include "gossip-stock.h"
 #include "gossip-app.h"
 #include "gossip-ui-utils.h"
 
-#define AVAILABLE_MESSAGE "Available"
-#define AWAY_MESSAGE "Away"
-#define BUSY_MESSAGE "Busy"
-
 struct SizeData {
 	gint     width;
 	gint     height;
 	gboolean preserve_aspect_ratio;
 };
 
+static void hint_dialog_response_cb             (GtkWidget       *widget,
+						 gint             response,
+						 GtkWidget       *checkbutton);
 static void pixbuf_from_avatar_size_prepared_cb (GdkPixbufLoader *loader,
 						 gint             width,
 						 gint             height,
 						 struct SizeData *data);
 
-static void
-tagify_bold_labels (GladeXML *xml)
-{
-        const gchar *str;
-        gchar       *s;
-        GtkWidget   *label;
-	GList       *labels, *l;
-
-	labels = glade_xml_get_widget_prefix (xml, "boldlabel");
-
-	for (l = labels; l; l = l->next) {
-		label = l->data;
-
-		if (!GTK_IS_LABEL (label)) {
-			g_warning ("Not a label, check your glade file.");
-			continue;
-		}
- 
-		str = gtk_label_get_text (GTK_LABEL (label));
-
-		s = g_strdup_printf ("<b>%s</b>", str);
-		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-		gtk_label_set_label (GTK_LABEL (label), s);
-		g_free (s);
-	}
-
-	g_list_free (labels);
-}
-
 static GladeXML *
 get_glade_file (const gchar *filename,
 		const gchar *root,
 		const gchar *domain,
-		const gchar *first_required_widget, 
+		const gchar *first_required_widget,
 		va_list      args)
 {
+	gchar      *path;
 	GladeXML   *gui;
 	const char *name;
 	GtkWidget **widget_ptr;
 
-	gui = glade_xml_new (filename, root, domain);
+	path = gossip_paths_get_glade_path (filename);
+	gui = glade_xml_new (path, root, domain);
+	g_free (path);
+
 	if (!gui) {
 		g_warning ("Couldn't find necessary glade file '%s'", filename);
 		return NULL;
@@ -99,9 +85,9 @@ get_glade_file (const gchar *filename,
 
 	for (name = first_required_widget; name; name = va_arg (args, char *)) {
 		widget_ptr = va_arg (args, void *);
-		
+
 		*widget_ptr = glade_xml_get_widget (gui, name);
-		
+
 		if (!*widget_ptr) {
 			g_warning ("Glade file '%s' is missing widget '%s'.",
 				   filename, name);
@@ -109,8 +95,6 @@ get_glade_file (const gchar *filename,
 		}
 	}
 
-	tagify_bold_labels (gui);
-	
 	return gui;
 }
 
@@ -130,7 +114,7 @@ gossip_glade_get_file_simple (const gchar *filename,
 			      domain,
 			      first_required_widget,
 			      args);
-	
+
 	va_end (args);
 
 	if (!gui) {
@@ -156,7 +140,7 @@ gossip_glade_get_file (const gchar *filename,
 			      domain,
 			      first_required_widget,
 			      args);
-	
+
 	va_end (args);
 
 	if (!gui) {
@@ -178,7 +162,7 @@ gossip_glade_connect (GladeXML *gui,
 	gpointer    *callback;
 
 	va_start (args, first_widget);
-	
+
 	for (name = first_widget; name; name = va_arg (args, char *)) {
 		signal = va_arg (args, void *);
 		callback = va_arg (args, void *);
@@ -212,7 +196,7 @@ gossip_glade_setup_size_group (GladeXML         *gui,
 	va_start (args, first_widget);
 
 	size_group = gtk_size_group_new (mode);
-	
+
 	for (name = first_widget; name; name = va_arg (args, char *)) {
 		widget = glade_xml_get_widget (gui, name);
 		if (!widget) {
@@ -235,18 +219,19 @@ password_dialog_activate_cb (GtkWidget *entry, GtkDialog *dialog)
 }
 
 gchar *
-gossip_password_dialog_run (GossipAccount *account, 
+gossip_password_dialog_run (GossipAccount *account,
 			    GtkWindow     *parent)
 {
-	GtkWidget *dialog;
-	GtkWidget *checkbox;
-	GtkWidget *entry;
-	GtkWidget *vbox;
-	gchar     *str;
-	gchar     *password;
+	GtkWidget   *dialog;
+	GtkWidget   *checkbox;
+	GtkWidget   *entry;
+	GtkWidget   *vbox;
+	gchar       *str;
+	gchar       *password;
+	const gchar *id;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
-	
+
 	str = g_strdup_printf (_("Please enter your %s account password"),
 			       gossip_account_get_name (account));
 	dialog = gtk_message_dialog_new_with_markup (parent,
@@ -257,43 +242,43 @@ gossip_password_dialog_run (GossipAccount *account,
 						     str);
 	g_free (str);
 
-	str = g_strdup_printf (_("Logging in with: %s"), 
-			       gossip_account_get_id (account));
+	gossip_account_param_get (account, "id", &id, NULL);
+	str = g_strdup_printf (_("Logging in with: %s"), id);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", str);
 	g_free (str);
 
 	checkbox = gtk_check_button_new_with_label (_("Remember Password?"));
 
 	entry = gtk_entry_new ();
-	gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE); 
-	
+	gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
+
 	g_signal_connect (entry,
 			  "activate",
 			  G_CALLBACK (password_dialog_activate_cb),
 			  dialog);
-	
+
 	vbox = gtk_vbox_new (FALSE, 6);
 
- 	gtk_container_set_border_width  (GTK_CONTAINER (vbox), 6); 
-	
+	gtk_container_set_border_width  (GTK_CONTAINER (vbox), 6);
+
 	gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), checkbox, FALSE, FALSE, 0);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, FALSE, FALSE, 0);
 
- 	gtk_widget_show_all (dialog); 
-	
+	gtk_widget_show_all (dialog);
+
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
 		password = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox))) {
 			GossipSession        *session;
 			GossipAccountManager *manager;
-			
+
 			session = gossip_app_get_session ();
 			manager = gossip_session_get_account_manager (session);
 
-			gossip_account_set_password (account, password);
+			gossip_account_param_set (account, "password", password, NULL);
 			gossip_account_manager_store (manager);
 		}
 	} else {
@@ -305,6 +290,105 @@ gossip_password_dialog_run (GossipAccount *account,
 	return password;
 }
 
+static void
+hint_dialog_response_cb (GtkWidget *widget,
+			 gint       response,
+			 GtkWidget *checkbutton)
+{
+	GFunc        func;
+	gpointer     user_data;
+	const gchar *conf_path;
+	gboolean     hide_hint;
+
+	conf_path = g_object_get_data (G_OBJECT (widget), "conf_path");
+	func = g_object_get_data (G_OBJECT (widget), "func");
+	user_data = g_object_get_data (G_OBJECT (widget), "user_data");
+
+	hide_hint = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
+	gossip_conf_set_bool (gossip_conf_get (), conf_path, !hide_hint);
+
+	gtk_widget_destroy (widget);
+
+	if (func) {
+		(func) ((gpointer) conf_path, user_data);
+	}
+}
+
+gboolean
+gossip_hint_dialog_show (const gchar *conf_path,
+			 const gchar *message1,
+			 const gchar *message2,
+			 GtkWindow   *parent,
+			 GFunc        func,
+			 gpointer     user_data)
+{
+	GtkWidget *dialog;
+	GtkWidget *checkbutton;
+	GtkWidget *vbox;
+	gboolean   ok;
+	gboolean   show_hint = TRUE;
+
+	g_return_val_if_fail (conf_path != NULL, FALSE);
+	g_return_val_if_fail (message1 != NULL, FALSE);
+
+	ok = gossip_conf_get_bool (gossip_conf_get (),
+				   conf_path,
+				   &show_hint);
+
+	if (ok && !show_hint) {
+		return FALSE;
+	}
+
+	dialog = gtk_message_dialog_new_with_markup (parent,
+						     GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_INFO,
+						     GTK_BUTTONS_CLOSE,
+						     "<b>%s</b>",
+						     message1);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  "%s", message2);
+	checkbutton = gtk_check_button_new_with_label (_("Do not show this again"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), TRUE);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width  (GTK_CONTAINER (vbox), 6);
+	gtk_box_pack_start (GTK_BOX (vbox), checkbutton, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, FALSE, FALSE, 0);
+
+	g_object_set_data_full (G_OBJECT (dialog), "conf_path", g_strdup (conf_path), g_free);
+	g_object_set_data (G_OBJECT (dialog), "user_data", user_data);
+	g_object_set_data (G_OBJECT (dialog), "func", func);
+
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (hint_dialog_response_cb),
+			  checkbutton);
+
+	gtk_widget_show_all (dialog);
+
+	return TRUE;
+}
+
+gboolean
+gossip_hint_show (const gchar         *conf_path,
+		  const gchar         *message1,
+		  const gchar         *message2,
+		  GtkWindow           *parent,
+		  GFunc                func,
+		  gpointer             user_data)
+{
+#ifdef HAVE_LIBNOTIFY
+	return gossip_notify_hint_show (conf_path,
+					message1, message2,
+					func, user_data);
+#else
+	return gossip_hint_dialog_show (conf_path,
+					message1, message2,
+					parent,
+					func, user_data);
+#endif
+}
+
 GdkPixbuf *
 gossip_pixbuf_from_stock (const gchar *stock,
 			  GtkIconSize  size)
@@ -321,7 +405,7 @@ gossip_pixbuf_from_account_type (GossipAccountType type,
 	GdkPixbuf     *pixbuf = NULL;
 	GError        *error = NULL;
 	gint           w, h;
-	gint           size = 48;  
+	gint           size = 48;
 	const gchar   *icon_id = NULL;
 
 	theme = gtk_icon_theme_get_default ();
@@ -329,7 +413,7 @@ gossip_pixbuf_from_account_type (GossipAccountType type,
 	if (!gtk_icon_size_lookup (icon_size, &w, &h)) {
 		size = 48;
 	} else {
-		size = (w + h) / 2; 
+		size = (w + h) / 2;
 	}
 
 	switch (type) {
@@ -394,7 +478,7 @@ gossip_pixbuf_from_account_status (GossipAccount *account,
 		modded_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 						TRUE,
 						8,
-						gdk_pixbuf_get_width (pixbuf), 
+						gdk_pixbuf_get_width (pixbuf),
 						gdk_pixbuf_get_height (pixbuf));
 
 		gdk_pixbuf_saturate_and_pixelate (pixbuf,
@@ -415,7 +499,7 @@ gossip_pixbuf_from_account_error (GossipAccount *account,
 	GdkPixbuf *pixbuf;
 	GdkPixbuf *pixbuf_error;
 
-	pixbuf = gossip_pixbuf_from_account_status (account, 
+	pixbuf = gossip_pixbuf_from_account_status (account,
 						    icon_size,
 						    FALSE);
 	if (!pixbuf) {
@@ -429,8 +513,8 @@ gossip_pixbuf_from_account_error (GossipAccount *account,
 		return NULL;
 	}
 
-	gdk_pixbuf_composite (pixbuf_error, 
-			      pixbuf, 
+	gdk_pixbuf_composite (pixbuf_error,
+			      pixbuf,
 			      0, 0,
 			      gdk_pixbuf_get_width (pixbuf_error),
 			      gdk_pixbuf_get_height (pixbuf_error),
@@ -452,7 +536,7 @@ gossip_pixbuf_from_smiley (GossipSmiley type,
 	GdkPixbuf     *pixbuf = NULL;
 	GError        *error = NULL;
 	gint           w, h;
-	gint           size;  
+	gint           size;
 	const gchar   *icon_id;
 
 	theme = gtk_icon_theme_get_default ();
@@ -460,7 +544,7 @@ gossip_pixbuf_from_smiley (GossipSmiley type,
 	if (!gtk_icon_size_lookup (icon_size, &w, &h)) {
 		size = 16;
 	} else {
-		size = (w + h) / 2; 
+		size = (w + h) / 2;
 	}
 
 	switch (type) {
@@ -527,26 +611,26 @@ gossip_pixbuf_from_smiley (GossipSmiley type,
 	case GOSSIP_SMILEY_BLUSH:        /*  *S   */
 		icon_id = "stock_smiley-23";
 		break;
-	case GOSSIP_SMILEY_COOLBIGSMILE: /*  8D   */ 
+	case GOSSIP_SMILEY_COOLBIGSMILE: /*  8D   */
 		icon_id = "stock_smiley-25";
 		break;
-	case GOSSIP_SMILEY_ANGRY:        /*  :@   */  
-		icon_id = "stock_smiley-16"; 
-		break; 
-	case GOSSIP_SMILEY_BOSS:         /*  @)   */  
-		icon_id = "stock_smiley-21"; 
-		break; 
-	case GOSSIP_SMILEY_MONKEY:       /*  #)   */  
-		icon_id = "stock_smiley-22"; 
-		break; 
-	case GOSSIP_SMILEY_SILLY:        /*  O)   */  
-		icon_id = "stock_smiley-24"; 
-		break; 
-	case GOSSIP_SMILEY_SICK:         /*  +o(  */  
-		icon_id = "stock_smiley-26"; 
-		break; 
+	case GOSSIP_SMILEY_ANGRY:        /*  :@   */
+		icon_id = "stock_smiley-16";
+		break;
+	case GOSSIP_SMILEY_BOSS:         /*  @)   */
+		icon_id = "stock_smiley-21";
+		break;
+	case GOSSIP_SMILEY_MONKEY:       /*  #)   */
+		icon_id = "stock_smiley-22";
+		break;
+	case GOSSIP_SMILEY_SILLY:        /*  O)   */
+		icon_id = "stock_smiley-24";
+		break;
+	case GOSSIP_SMILEY_SICK:         /*  +o(  */
+		icon_id = "stock_smiley-26";
+		break;
 
-	default: 
+	default:
 		g_assert_not_reached ();
 		icon_id = NULL;
 	}
@@ -563,7 +647,7 @@ gossip_pixbuf_from_smiley (GossipSmiley type,
 GdkPixbuf *
 gossip_pixbuf_offline (void)
 {
-	return gossip_pixbuf_from_stock (GOSSIP_STOCK_OFFLINE, 
+	return gossip_pixbuf_from_stock (GOSSIP_STOCK_OFFLINE,
 					 GTK_ICON_SIZE_MENU);
 }
 
@@ -596,10 +680,12 @@ gossip_window_get_is_visible (GtkWindow *window)
 
 /* Takes care of moving the window to the current workspace. */
 void
-gossip_window_present (GtkWindow *window)
+gossip_window_present (GtkWindow *window,
+		       gboolean   steal_focus)
 {
 	gboolean visible;
 	gboolean on_current;
+	guint32  timestamp;
 
 	/* There are three cases: hidden, visible, visible on another
 	 * workspace.
@@ -616,13 +702,18 @@ gossip_window_present (GtkWindow *window)
 		gtk_widget_hide (GTK_WIDGET (window));
 	}
 
-	gtk_window_present (window);
+	timestamp = gtk_get_current_event_time ();
+	if (steal_focus && timestamp != GDK_CURRENT_TIME) {
+		gtk_window_present_with_time (window, timestamp);
+	} else {
+		gtk_window_present (window);
+	}
 }
 
 GdkPixbuf *
 gossip_pixbuf_for_presence_state (GossipPresenceState state)
 {
-	const gchar *stock = NULL; 
+	const gchar *stock = NULL;
 
 	switch (state) {
 	case GOSSIP_PRESENCE_STATE_AVAILABLE:
@@ -649,7 +740,7 @@ gossip_pixbuf_for_presence_state (GossipPresenceState state)
 GdkPixbuf *
 gossip_pixbuf_for_presence (GossipPresence *presence)
 {
-	GossipPresenceState state; 
+	GossipPresenceState state;
 
 	g_return_val_if_fail (GOSSIP_IS_PRESENCE (presence),
 			      gossip_pixbuf_offline ());
@@ -659,13 +750,13 @@ gossip_pixbuf_for_presence (GossipPresence *presence)
 	return gossip_pixbuf_for_presence_state (state);
 }
 
-GdkPixbuf *  
+GdkPixbuf *
 gossip_pixbuf_for_contact (GossipContact *contact)
 {
 	GossipPresence     *presence;
 	GossipSubscription  subscription;
 
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), 
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact),
 			      gossip_pixbuf_offline ());
 
 	presence = gossip_contact_get_active_presence (contact);
@@ -676,9 +767,9 @@ gossip_pixbuf_for_contact (GossipContact *contact)
 
 	subscription = gossip_contact_get_subscription (contact);
 
-	if (subscription != GOSSIP_SUBSCRIPTION_BOTH && 
+	if (subscription != GOSSIP_SUBSCRIPTION_BOTH &&
 	    subscription != GOSSIP_SUBSCRIPTION_TO) {
-		return gossip_pixbuf_from_stock (GOSSIP_STOCK_PENDING, 
+		return gossip_pixbuf_from_stock (GOSSIP_STOCK_PENDING,
 						 GTK_ICON_SIZE_MENU);
 	}
 
@@ -696,7 +787,7 @@ gossip_pixbuf_for_chatroom_status (GossipChatroom *chatroom,
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
 
 	status = gossip_chatroom_get_status (chatroom);
-	
+
 	switch (status) {
 	case GOSSIP_CHATROOM_STATUS_JOINING:
 		stock_id = GTK_STOCK_EXECUTE;
@@ -706,7 +797,7 @@ gossip_pixbuf_for_chatroom_status (GossipChatroom *chatroom,
 		break;
 	case GOSSIP_CHATROOM_STATUS_ERROR:
 		stock_id = GTK_STOCK_DIALOG_ERROR;
-		break;	
+		break;
 	default:
 	case GOSSIP_CHATROOM_STATUS_INACTIVE:
 	case GOSSIP_CHATROOM_STATUS_UNKNOWN:
@@ -720,15 +811,16 @@ gossip_pixbuf_for_chatroom_status (GossipChatroom *chatroom,
 }
 
 GdkPixbuf *
-gossip_pixbuf_avatar_from_vcard (GossipVCard *vcard) 
+gossip_pixbuf_avatar_from_vcard (GossipVCard *vcard)
 {
 	GdkPixbuf	*pixbuf;
 	GdkPixbufLoader	*loader;
 	const guchar    *avatar;
 	gsize		 len;
-	
+	GError          *error = NULL;
+
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	
+
 	avatar = gossip_vcard_get_avatar (vcard, &len);
 	if (!avatar) {
 		return NULL;
@@ -736,10 +828,11 @@ gossip_pixbuf_avatar_from_vcard (GossipVCard *vcard)
 
 	loader = gdk_pixbuf_loader_new ();
 
-	if (!gdk_pixbuf_loader_write (loader, avatar, len, NULL)) {
+	if (!gdk_pixbuf_loader_write (loader, avatar, len, &error)) {
 		g_warning ("Couldn't write avatar image:%p with "
-			   "length:%" G_GSIZE_FORMAT " to pixbuf loader",
-			   avatar, len);
+			   "length:%" G_GSIZE_FORMAT " to pixbuf loader: %s",
+			   avatar, len, error->message);
+		g_error_free (error);
 		return NULL;
 	}
 
@@ -754,15 +847,16 @@ gossip_pixbuf_avatar_from_vcard (GossipVCard *vcard)
 }
 
 GdkPixbuf *
-gossip_pixbuf_avatar_from_contact (GossipContact *contact) 
+gossip_pixbuf_avatar_from_contact (GossipContact *contact)
 {
 	GdkPixbuf	*pixbuf;
 	GdkPixbufLoader	*loader;
 	const guchar	*avatar;
 	gsize		 len;
-	
+	GError          *error = NULL;
+
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
-	
+
 	avatar = gossip_contact_get_avatar (contact, &len);
 	if (!avatar) {
 		return NULL;
@@ -770,10 +864,11 @@ gossip_pixbuf_avatar_from_contact (GossipContact *contact)
 
 	loader = gdk_pixbuf_loader_new ();
 
-	if (!gdk_pixbuf_loader_write (loader, avatar, len, NULL)) {
+	if (!gdk_pixbuf_loader_write (loader, avatar, len, &error)) {
 		g_warning ("Couldn't write avatar image:%p with "
-			   "length:%" G_GSIZE_FORMAT " to pixbuf loader",
-			   avatar, len);
+			   "length:%" G_GSIZE_FORMAT " to pixbuf loader: %s",
+			   avatar, len, error->message);
+		g_error_free (error);
 		return NULL;
 	}
 
@@ -829,7 +924,7 @@ pixbuf_from_avatar_size_prepared_cb (GdkPixbufLoader *loader,
 }
 
 GdkPixbuf *
-gossip_pixbuf_from_avatar_scaled (const guchar *avatar, 
+gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
 				  gsize         len,
 				  gint          width,
 				  gint          height)
@@ -837,6 +932,7 @@ gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
 	GdkPixbuf        *pixbuf;
 	GdkPixbufLoader	 *loader;
 	struct SizeData   data;
+	GError           *error = NULL;
 
 	if (!avatar) {
 		return NULL;
@@ -848,14 +944,15 @@ gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
 
 	loader = gdk_pixbuf_loader_new ();
 
-	g_signal_connect (loader, "size-prepared", 
-			  G_CALLBACK (pixbuf_from_avatar_size_prepared_cb), 
+	g_signal_connect (loader, "size-prepared",
+			  G_CALLBACK (pixbuf_from_avatar_size_prepared_cb),
 			  &data);
 
-	if (!gdk_pixbuf_loader_write (loader, avatar, len, NULL)) {
+	if (!gdk_pixbuf_loader_write (loader, avatar, len, &error)) {
 		g_warning ("Couldn't write avatar image:%p with "
-			   "length:%" G_GSIZE_FORMAT " to pixbuf loader",
-			   avatar, len);
+			   "length:%" G_GSIZE_FORMAT " to pixbuf loader: %s",
+			   avatar, len, error->message);
+		g_error_free (error);
 		return NULL;
 	}
 
@@ -870,7 +967,7 @@ gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
 }
 
 GdkPixbuf *
-gossip_pixbuf_avatar_from_contact_scaled (GossipContact *contact, 
+gossip_pixbuf_avatar_from_contact_scaled (GossipContact *contact,
 					  gint           width,
 					  gint           height)
 {
@@ -883,18 +980,18 @@ gossip_pixbuf_avatar_from_contact_scaled (GossipContact *contact,
 	if (!avatar) {
 		return NULL;
 	}
-	
+
 	return gossip_pixbuf_from_avatar_scaled (avatar, len, width, height);
 }
 
 GdkPixbuf *
-gossip_pixbuf_avatar_from_vcard_scaled (GossipVCard *vcard, 
+gossip_pixbuf_avatar_from_vcard_scaled (GossipVCard *vcard,
 					GtkIconSize  size)
 {
 	const guchar *avatar;
 	gsize	      len;
 	gint          width, height;
-	
+
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
 
 	avatar = gossip_vcard_get_avatar (vcard, &len);
@@ -1027,7 +1124,7 @@ g_utf8_caselessnmatch (const char *s1, const char *s2,
 
 finally_2:
 	g_free (normalized_s1);
-	g_free (normalized_s2);	
+	g_free (normalized_s2);
 
 	return ret;
 }
@@ -1066,7 +1163,7 @@ forward_chars_with_skipping (GtkTextIter *iter,
 		{
 			/* being UTF8 correct sucks; this accounts for extra
 			   offsets coming from canonical decompositions of
-			   UTF8 characters (e.g. accented characters) which 
+			   UTF8 characters (e.g. accented characters) which
 			   g_utf8_normalize() performs */
 			gchar *normal;
 			gchar buffer[6];
@@ -1283,7 +1380,7 @@ gossip_text_iter_forward_search (const GtkTextIter   *iter,
 			return FALSE;
 		}
 	}
-	
+
 	visible_only = TRUE;
 	slice = FALSE;
 
@@ -1324,3 +1421,403 @@ gossip_text_iter_forward_search (const GtkTextIter   *iter,
 
 	return retval;
 }
+
+static const gchar *
+g_utf8_strrcasestr (const gchar *haystack, const gchar *needle)
+{
+	gsize needle_len;
+	gsize haystack_len;
+	const gchar *ret = NULL;
+	gchar *p;
+	gchar *casefold;
+	gchar *caseless_haystack;
+	gint i;
+
+	g_return_val_if_fail (haystack != NULL, NULL);
+	g_return_val_if_fail (needle != NULL, NULL);
+
+	casefold = g_utf8_casefold (haystack, -1);
+	caseless_haystack = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	needle_len = g_utf8_strlen (needle, -1);
+	haystack_len = g_utf8_strlen (caseless_haystack, -1);
+
+	if (needle_len == 0)
+	{
+		ret = (gchar *)haystack;
+		goto finally_1;
+	}
+
+	if (haystack_len < needle_len)
+	{
+		ret = NULL;
+		goto finally_1;
+	}
+
+	i = haystack_len - needle_len;
+	p = g_utf8_offset_to_pointer (caseless_haystack, i);
+	needle_len = strlen (needle);
+
+	while (p >= caseless_haystack)
+	{
+		if (strncmp (p, needle, needle_len) == 0)
+		{
+			ret = pointer_from_offset_skipping_decomp (haystack, i);
+			goto finally_1;
+		}
+
+		p = g_utf8_prev_char (p);
+		i--;
+	}
+
+finally_1:
+	g_free (caseless_haystack);
+
+	return ret;
+}
+
+static gboolean
+backward_lines_match (const GtkTextIter *start,
+		      const gchar      **lines,
+		      gboolean           visible_only,
+		      gboolean           slice,
+		      GtkTextIter       *match_start,
+		      GtkTextIter       *match_end)
+{
+	GtkTextIter line, next;
+	gchar *line_text;
+	const gchar *found;
+	gint offset;
+
+	if (*lines == NULL || **lines == '\0')
+	{
+		if (match_start)
+			*match_start = *start;
+		if (match_end)
+			*match_end = *start;
+		return TRUE;
+	}
+
+	line = next = *start;
+	if (gtk_text_iter_get_line_offset (&next) == 0)
+	{
+		if (!gtk_text_iter_backward_line (&next))
+			return FALSE;
+	}
+	else
+		gtk_text_iter_set_line_offset (&next, 0);
+
+	if (slice)
+	{
+		if (visible_only)
+			line_text = gtk_text_iter_get_visible_slice (&next, &line);
+		else
+			line_text = gtk_text_iter_get_slice (&next, &line);
+	}
+	else
+	{
+		if (visible_only)
+			line_text = gtk_text_iter_get_visible_text (&next, &line);
+		else
+			line_text = gtk_text_iter_get_text (&next, &line);
+	}
+
+	if (match_start) /* if this is the first line we're matching */
+	{
+		found = g_utf8_strrcasestr (line_text, *lines);
+	}
+	else
+	{
+		/* If it's not the first line, we have to match from the
+		 * start of the line.
+		 */
+		if (g_utf8_caselessnmatch (line_text, *lines, strlen (line_text),
+					   strlen (*lines)))
+			found = line_text;
+		else
+			found = NULL;
+	}
+
+	if (found == NULL)
+	{
+		g_free (line_text);
+		return FALSE;
+	}
+
+	/* Get offset to start of search string */
+	offset = g_utf8_strlen (line_text, found - line_text);
+
+	forward_chars_with_skipping (&next, offset, visible_only, !slice, FALSE);
+
+	/* If match start needs to be returned, set it to the
+	 * start of the search string.
+	 */
+	if (match_start)
+	{
+		*match_start = next;
+	}
+
+	/* Go to end of search string */
+	forward_chars_with_skipping (&next, g_utf8_strlen (*lines, -1), visible_only, !slice, TRUE);
+
+	g_free (line_text);
+
+	++lines;
+
+	if (match_end)
+		*match_end = next;
+
+	/* try to match the rest of the lines forward, passing NULL
+	 * for match_start so lines_match will try to match the entire
+	 * line */
+	return lines_match (&next, lines, visible_only,
+			    slice, NULL, match_end);
+}
+
+gboolean
+gossip_text_iter_backward_search (const GtkTextIter   *iter,
+				  const gchar         *str,
+				  GtkTextIter         *match_start,
+				  GtkTextIter         *match_end,
+				  const GtkTextIter   *limit)
+{
+	gchar **lines = NULL;
+	GtkTextIter match;
+	gboolean retval = FALSE;
+	GtkTextIter search;
+	gboolean visible_only;
+	gboolean slice;
+
+	g_return_val_if_fail (iter != NULL, FALSE);
+	g_return_val_if_fail (str != NULL, FALSE);
+
+	if (limit && gtk_text_iter_compare (iter, limit) <= 0)
+		return FALSE;
+
+	if (*str == '\0')
+	{
+		/* If we can move one char, return the empty string there */
+		match = *iter;
+
+		if (gtk_text_iter_backward_char (&match))
+		{
+			if (limit && gtk_text_iter_equal (&match, limit))
+				return FALSE;
+
+			if (match_start)
+				*match_start = match;
+			if (match_end)
+				*match_end = match;
+			return TRUE;
+		}
+		else
+		{
+			return FALSE;
+		}
+	}
+
+	visible_only = TRUE;
+	slice = TRUE;
+
+	/* locate all lines */
+	lines = strbreakup (str, "\n", -1);
+
+	search = *iter;
+
+	while (TRUE)
+	{
+		/* This loop has an inefficient worst-case, where
+		 * gtk_text_iter_get_text () is called repeatedly on
+		 * a single line.
+		 */
+		GtkTextIter end;
+
+		if (limit && gtk_text_iter_compare (&search, limit) <= 0)
+			break;
+
+		if (backward_lines_match (&search, (const gchar**)lines,
+					  visible_only, slice, &match, &end))
+		{
+			if (limit == NULL || (limit &&
+					      gtk_text_iter_compare (&end, limit) > 0))
+			{
+				retval = TRUE;
+
+				if (match_start)
+					*match_start = match;
+				if (match_end)
+					*match_end = end;
+			}
+			break;
+		}
+
+		if (gtk_text_iter_get_line_offset (&search) == 0)
+		{
+			if (!gtk_text_iter_backward_line (&search))
+				break;
+		}
+		else
+		{
+			gtk_text_iter_set_line_offset (&search, 0);
+		}
+	}
+
+	g_strfreev ((gchar**)lines);
+
+	return retval;
+}
+
+/* The URL opening code can't handle schemeless strings, so we try to be
+ * smart and add http if there is no scheme or doesn't look like a mail
+ * address. This should work in most cases, and let us click on strings
+ * like "www.gnome.org".
+ */
+static gchar *
+fixup_url (const gchar *url)
+{
+	gchar *real_url;
+
+	if (!g_str_has_prefix (url, "http://") &&
+	    !strstr (url, ":/") &&
+	    !strstr (url, "@")) {
+		real_url = g_strdup_printf ("http://%s", url);
+	} else {
+		real_url = g_strdup (url);
+	}
+
+	return real_url;
+}
+
+#ifdef HAVE_GNOME
+void
+gossip_url_show (const char *url)
+{
+	gchar  *real_url;
+	GError *error = NULL;
+
+	real_url = fixup_url (url);
+	gnome_url_show (real_url, &error);
+	if (error) {
+		g_warning ("Couldn't show URL:'%s'", real_url);
+		g_error_free (error);
+	}
+
+	g_free (real_url);
+}
+#elif defined(HAVE_COCOA)
+void
+gossip_url_show (const char *url)
+{
+	gchar             *real_url;
+	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+	NSString          *string;
+
+	real_url = fixup_url (url);
+
+	string = [NSString stringWithUTF8String: real_url];
+	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:string]];
+
+	[pool release];
+
+	g_free (real_url);
+}
+#else
+void
+gossip_url_show (const char *url)
+{
+}
+#endif
+
+void
+gossip_help_show (void)
+{
+#ifdef HAVE_GNOME
+	gboolean   ok;
+	GtkWidget *dialog;
+	GError    *err = NULL;
+
+	ok = gnome_help_display ("gossip.xml", NULL, &err);
+	if (ok) {
+		return;
+	}
+
+	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (gossip_app_get_window ()),
+						     GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     "<b>%s</b>\n\n%s",
+						     _("Could not display the help contents."),
+						     err->message);
+
+	g_signal_connect_swapped (dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  dialog);
+
+	gtk_widget_show (dialog);
+
+	g_error_free (err);
+#elif defined(HAVE_COCOA)
+	/* Nothing for now. */
+#endif
+}
+
+static void
+link_button_hook (GtkLinkButton *button,
+		  const gchar *link,
+		  gpointer user_data)
+{
+	gossip_url_show (link);
+}
+
+GtkWidget *
+gossip_link_button_new (const gchar *url,
+			const gchar *title)
+{
+	static gboolean hook = FALSE;
+
+	if (!hook) {
+		hook = TRUE;
+		gtk_link_button_set_uri_hook (link_button_hook, NULL, NULL);
+	}
+
+	return gtk_link_button_new_with_label (url, title);
+}
+
+/* FIXME: Do this in a proper way at some point, probably in GTK+? */
+void
+gossip_window_set_default_icon_name (const gchar *name)
+{
+#ifdef HAVE_GNOME
+	gtk_window_set_default_icon_name (name);
+#elif defined(HAVE_COCOA)
+	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+	gchar             *path;
+	NSString          *tmp;
+	NSImage           *image;
+
+	path = gossip_paths_get_image_path ("gossip-logo.png");
+
+	tmp = [NSString stringWithUTF8String:path];
+	image = [[NSImage alloc] initWithContentsOfFile:tmp];
+	[NSApp setApplicationIconImage:image];
+	[image release];
+
+	g_free (path);
+
+	[pool release];
+#endif
+}
+
+void
+gossip_request_user_attention (void)
+{
+#ifdef HAVE_GNOME
+	/* Nothing for now. */
+#elif defined(HAVE_COCOA)
+	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+	[NSApp requestUserAttention: NSInformationalRequest];
+	[pool release];
+#endif
+}
+
